@@ -1,8 +1,8 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python3
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -22,8 +22,6 @@
 Launcher
 """
 
-from __future__ import absolute_import, print_function
-
 import os
 import sys
 
@@ -33,6 +31,13 @@ if 'posix' == os.name:
         # unnecessary if installing BleachBit in site-packages.
         sys.path.append('/usr/share/')
 
+if os.name == 'nt':
+    # change error handling to avoid popup with GTK 3
+    # https://github.com/bleachbit/bleachbit/issues/651
+    import win32api
+    import win32con
+    win32api.SetErrorMode(win32con.SEM_FAILCRITICALERRORS |
+                          win32con.SEM_NOGPFAULTERRORBOX | win32con.SEM_NOOPENFILEERRORBOX)
 
 if 1 == len(sys.argv):
     import bleachbit.GUI
