@@ -2,7 +2,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2019 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -22,10 +22,8 @@
 Command line interface
 """
 
-from __future__ import absolute_import, print_function
-
 from bleachbit.Cleaner import backends, create_simple_cleaner, register_cleaners
-from bleachbit import _, APP_VERSION, encoding
+from bleachbit import _, APP_VERSION
 from bleachbit import Diagnostic, Options, Worker
 
 import logging
@@ -39,16 +37,9 @@ logger = logging.getLogger(__name__)
 class CliCallback:
     """Command line's callback passed to Worker"""
 
-    def __init__(self):
-        """Initialize CliCallback"""
-        self.encoding = encoding if encoding else 'UTF8'
-
     def append_text(self, msg, tag=None):
         """Write text to the terminal"""
-        # If the encoding is not explicitly handled on a non-UTF-8
-        # system, then special Latin-1 characters such as umlauts may
-        # raise an exception as an encoding error.
-        print(msg.strip('\n').encode(self.encoding, 'replace'))
+        print(msg.strip('\n'))
 
     def update_progress_bar(self, status):
         """Not used"""
@@ -86,7 +77,7 @@ def preview_or_clean(operations, really_clean):
     """Preview deletes and other changes"""
     cb = CliCallback()
     worker = Worker.Worker(cb, really_clean, operations).run()
-    while worker.next():
+    while next(worker):
         pass
 
 
@@ -140,7 +131,8 @@ def process_cmd_line():
                       # This is different than cleaning an arbitrary file, such as a
                       # spreadsheet on the desktop.
                       help=_("run cleaners to delete files and make other permanent changes"))
-    parser.add_option('--debug', help=_("set log level to verbose"), action="store_true")
+    parser.add_option(
+        '--debug', help=_("set log level to verbose"), action="store_true")
     parser.add_option('--debug-log', help=_("log debug messages to file"))
     parser.add_option("-s", "--shred", action="store_true",
                       help=_("shred specific files or folders"))
@@ -190,7 +182,7 @@ def process_cmd_line():
     if options.version:
         print("""
 BleachBit version %s
-Copyright (C) 2008-2019 Andrew Ziem.  All rights reserved.
+Copyright (C) 2008-2020 Andrew Ziem.  All rights reserved.
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION)
@@ -244,7 +236,8 @@ There is NO WARRANTY, to the extent permitted by law.""" % APP_VERSION)
         sys.exit(0)
     if options.gui:
         import bleachbit.GUI
-        app = bleachbit.GUI.Bleachbit(uac=not options.no_uac, shred_paths=args, auto_exit=options.exit)
+        app = bleachbit.GUI.Bleachbit(
+            uac=not options.no_uac, shred_paths=args, auto_exit=options.exit)
         sys.exit(app.run())
     if options.shred:
         # delete arbitrary files without GUI
