@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -17,38 +17,40 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-
 """
 Perform (or assist with) cleaning operations.
 """
 
-from __future__ import absolute_import, print_function
-
-from bleachbit import _, expanduser, expandvars
-from bleachbit.FileUtilities import children_in_directory
-from bleachbit.Options import options
-from bleachbit import Command, FileUtilities, Memory, Special
-
 import glob
 import logging
 import os.path
 import re
 import sys
+
+from bleachbit import _
+from bleachbit.FileUtilities import children_in_directory
+from bleachbit.Options import options
+from bleachbit import Command, FileUtilities, Memory, Special
+
+
+# Suppress GTK warning messages while running in CLI #34
 import warnings
+warnings.simplefilter("ignore", Warning)
+try:
+    from bleachbit.GuiBasic import Gtk, Gdk
+    HAVE_GTK = Gdk.get_default_root_window() is not None
+except (ImportError, RuntimeError, ValueError) as e:
+    # ImportError happens when GTK is not installed.
+    # RuntimeError can happen when X is not available (e.g., cron, ssh).
+    # ValueError seen on BleachBit 3.0 with GTK 3 (GitHub issue 685)
+    HAVE_GTK = False
+
 
 if 'posix' == os.name:
     from bleachbit import Unix
 elif 'nt' == os.name:
     from bleachbit import Windows
 
-# Suppress GTK warning messages while running in CLI #34
-warnings.simplefilter("ignore", Warning)
-
-try:
-    import gtk
-    HAVE_GTK = True
-except ImportError:
-    HAVE_GTK = False
 
 # a module-level variable for holding cleaners
 backends = {}
@@ -66,6 +68,7 @@ class Cleaner:
         self.options = {}
         self.running = []
         self.warnings = {}
+        self.regexes_compiled = []
 
     def add_action(self, option_id, action):
         """Register 'action' (instance of class Action) to be executed
@@ -111,8 +114,11 @@ class Cleaner:
         """Get dictionary used to build a deep scan"""
         for action in self.actions:
             if option_id == action[0]:
-                for ds in action[1].get_deep_scan():
-                    yield ds
+                try:
+                    for ds in action[1].get_deep_scan():
+                        yield ds
+                except StopIteration:
+                    return
         if option_id not in self.options:
             raise RuntimeError("Unknown option '%s'" % option_id)
 
@@ -162,7 +168,7 @@ class Cleaner:
                     logger.debug("process '%s' is running", pathname)
                     return True
             elif 'pathname' == test:
-                expanded = expanduser(expandvars(pathname))
+                expanded = os.path.expanduser(os.path.expandvars(pathname))
                 for globbed in glob.iglob(expanded):
                     if os.path.exists(globbed):
                         logger.debug(
@@ -182,185 +188,6 @@ class Cleaner:
         self.warnings[option_id] = description
 
 
-class Firefox(Cleaner):
-
-    """Mozilla Firefox"""
-
-    def __init__(self):
-        Cleaner.__init__(self)
-        self.add_option('backup', _('Backup files'), _(
-            'Delete the backup files'))
-        self.add_option('cache', _('Cache'), _(
-            'Delete the web cache, which reduces time to display revisited pages'))
-        self.add_option('cookies', _('Cookies'), _(
-            'Delete cookies, which contain information such as web site preferences, authentication, and tracking identification'))
-        self.add_option(
-            'crash_reports', _('Crash reports'), _('Delete the files'))
-        # TRANSLATORS: DOM = Document Object Model.
-        self.add_option('dom', _('DOM Storage'), _('Delete HTML5 cookies'))
-        self.add_option('download_history', _(
-            'Download history'), _('List of files downloaded'))
-        self.add_option('forms', _('Form history'), _(
-            'A history of forms entered in web sites and in the Search bar'))
-        self.add_option('session_restore', _('Session restore'), _(
-            'Loads the initial session after the browser closes or crashes'))
-        self.add_option('site_preferences', _(
-            'Site preferences'), _('Settings for individual sites'))
-        self.add_option('passwords', _('Passwords'), _(
-            'A database of usernames and passwords as well as a list of sites that should not store passwords'))
-        self.set_warning(
-            'passwords', _('This option will delete your saved passwords.'))
-        self.add_option(
-            'url_history', _('URL history'), _('List of visited web pages'))
-        self.add_option('vacuum', _('Vacuum'), _(
-            'Clean database fragmentation to reduce space and improve speed without removing any data'))
-
-        if 'posix' == os.name:
-            self.profile_dir = "~/.mozilla/firefox*/*.default*/"
-            self.add_running('exe', 'firefox')
-            self.add_running('exe', 'firefox-bin')
-            self.add_running('pathname', self.profile_dir + 'lock')
-        elif 'nt' == os.name:
-            self.profile_dir = "%APPDATA%\\Mozilla\\Firefox\\Profiles\\*.default*\\"
-            self.add_running('exe', 'firefox.exe')
-
-        self.description = _("Web browser")
-        self.id = 'firefox'
-        self.name = "Firefox"
-
-    def get_commands(self, option_id):
-        files = []
-        # backup files
-        if 'backup' == option_id:
-            bookmark_bu_dir = os.path.join(self.profile_dir, 'bookmarkbackups')
-            files += FileUtilities.expand_glob_join(bookmark_bu_dir, "*.json")
-            files += FileUtilities.expand_glob_join(
-                bookmark_bu_dir, "*.jsonlz4")
-        # browser cache
-        cache_base = None
-        if 'posix' == os.name:
-            cache_base = self.profile_dir
-        elif 'nt' == os.name:
-            cache_base = "$localappdata\\Mozilla\\Firefox\\Profiles\\*.default*"
-        if 'cache' == option_id:
-            dirs = FileUtilities.expand_glob_join(cache_base, "Cache*")
-            dirs += FileUtilities.expand_glob_join(cache_base, "OfflineCache")
-            if 'nt' == os.name:
-                dirs += FileUtilities.expand_glob_join(
-                    cache_base, "jumpListCache")  # Windows 8
-            if 'posix' == os.name:
-                # This path is whitelisted under the System - Cache cleaner,
-                # so it can be cleaned here.
-                dirs += [expanduser('~/.cache/mozilla')]
-            for dirname in dirs:
-                for filename in children_in_directory(dirname, False):
-                    yield Command.Delete(filename)
-            # Necko Predictive Network Actions
-            # https://wiki.mozilla.org/Privacy/Reviews/Necko
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "netpredictions.sqlite")
-        # cookies
-        if 'cookies' == option_id:
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "cookies.txt")
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "cookies.sqlite")
-        # crash reports
-        if 'posix' == os.name:
-            crashdir = expanduser("~/.mozilla/firefox/Crash Reports")
-        if 'nt' == os.name:
-            crashdir = expandvars(
-                "$USERPROFILE\\Application Data\\Mozilla\\Firefox\\Crash Reports")
-        if 'crash_reports' == option_id:
-            for filename in children_in_directory(crashdir, False):
-                files += [filename]
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "minidumps/*.dmp")
-        # DOM storage
-        if 'dom' == option_id:
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "webappsstore.sqlite")
-
-        # download history
-        if 'download_history' == option_id:
-            # Firefox version 1
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "downloads.rdf")
-            # Firefox version 3
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "downloads.sqlite")
-        # forms
-        if 'forms' == option_id:
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "formhistory.dat")
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "formhistory.sqlite")
-        # passwords
-        if 'passwords' == option_id:
-            # http://kb.mozillazine.org/Password_Manager
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "signons.txt")
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "signons[2-3].txt")
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "signons.sqlite")
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "logins.json")
-        # session restore
-        if 'session_restore' == option_id:
-            # Names include sessionstore.js, sessionstore.bak,
-            # sessionstore.bak-20140715214327, sessionstore-1.js
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "sessionstore*.js")
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "sessionstore.bak*")
-            ss_bu_dir = os.path.join(self.profile_dir, 'sessionstore-backups')
-            files += FileUtilities.expand_glob_join(
-                ss_bu_dir, 'previous.js')
-            files += FileUtilities.expand_glob_join(
-                ss_bu_dir, 'upgrade.js-20*')
-            files += FileUtilities.expand_glob_join(
-                ss_bu_dir, 'recovery.js')
-            files += FileUtilities.expand_glob_join(
-                ss_bu_dir, 'recovery.bak')
-        # site-specific preferences
-        if 'site_preferences' == option_id:
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "content-prefs.sqlite")
-
-        # URL history
-        if 'url_history' == option_id:
-            # Firefox version 1
-            files += FileUtilities.expand_glob_join(
-                self.profile_dir, "history.dat")
-            # Firefox 21 on Windows
-            if 'nt' == os.name:
-                files += FileUtilities.expand_glob_join(
-                    cache_base, "thumbnails/*.png")
-            # see also function other_cleanup()
-        # finish
-        for filename in files:
-            yield Command.Delete(filename)
-
-        # URL history
-        if 'url_history' == option_id:
-            for path in FileUtilities.expand_glob_join(self.profile_dir, "places.sqlite"):
-                yield Command.Function(path,
-                                       Special.delete_mozilla_url_history,
-                                       _('Delete the usage history'))
-
-        # vacuum
-        if 'vacuum' == option_id:
-            paths = []
-            paths += FileUtilities.expand_glob_join(
-                self.profile_dir, "*.sqlite")
-            if not cache_base == self.profile_dir:
-                paths += FileUtilities.expand_glob_join(cache_base, "*.sqlite")
-            for path in paths:
-                yield Command.Function(path,
-                                       FileUtilities.vacuum_sqlite3, _("Vacuum"))
-
-
 class OpenOfficeOrg(Cleaner):
 
     """Delete OpenOffice.org cache"""
@@ -486,7 +313,7 @@ class System(Cleaner):
         if 'nt' == os.name:
             self.add_option('logs', _('Logs'), _('Delete the logs'))
             self.add_option(
-                'memory_dump', _('Memory dump'), _('Delete the file memory.dmp'))
+                'memory_dump', _('Memory dump'), _('Delete the file'))
             self.add_option('muicache', 'MUICache', _('Delete the cache'))
             # TRANSLATORS: Prefetch is Microsoft Windows jargon.
             self.add_option('prefetch', _('Prefetch'), _('Delete the cache'))
@@ -525,28 +352,22 @@ class System(Cleaner):
         self.name = _("System")
 
     def get_commands(self, option_id):
-        # This variable will collect fully expanded file names, and
-        # at the end of this function, they will be checked they exist
-        # and processed through Command.Delete().
-        files = []
-
         # cache
         if 'posix' == os.name and 'cache' == option_id:
-            dirname = expanduser("~/.cache/")
+            dirname = os.path.expanduser("~/.cache/")
             for filename in children_in_directory(dirname, True):
-                if self.whitelisted(filename):
-                    continue
-                files += [filename]
+                if not self.whitelisted(filename):
+                    yield Command.Delete(filename)
 
         # custom
         if 'custom' == option_id:
             for (c_type, c_path) in options.get_custom_paths():
                 if 'file' == c_type:
-                    files += [c_path]
+                    yield Command.Delete(c_path)
                 elif 'folder' == c_type:
-                    files += [c_path]
                     for path in children_in_directory(c_path, True):
-                        files += [path]
+                        yield Command.Delete(path)
+                    yield Command.Delete(c_path)
                 else:
                     raise RuntimeError(
                         'custom folder has invalid type %s' % c_type)
@@ -619,9 +440,9 @@ class System(Cleaner):
                 '$windir\\system32\\wbem\\Logs\\*.log', )
 
             for path in paths:
-                expanded = expandvars(path)
+                expanded = os.path.expandvars(path)
                 for globbed in glob.iglob(expanded):
-                    files += [globbed]
+                    yield Command.Delete(globbed)
 
         # memory
         if sys.platform.startswith('linux') and 'memory' == option_id:
@@ -631,15 +452,17 @@ class System(Cleaner):
         # how to manually create this file
         # http://www.pctools.com/guides/registry/detail/856/
         if 'nt' == os.name and 'memory_dump' == option_id:
-            fname = expandvars('$windir\\memory.dmp')
+            fname = os.path.expandvars('$windir\\memory.dmp')
             if os.path.exists(fname):
-                files += [fname]
-            for fname in glob.iglob(expandvars('$windir\\Minidump\\*.dmp')):
-                files += [fname]
+                yield Command.Delete(fname)
+            for fname in glob.iglob(os.path.expandvars('$windir\\Minidump\\*.dmp')):
+                yield Command.Delete(fname)
 
         # most recently used documents list
         if 'posix' == os.name and 'recent_documents' == option_id:
-            files += [expanduser("~/.recently-used")]
+            ru_fn = os.path.expanduser("~/.recently-used")
+            if os.path.lexists(ru_fn):
+                yield Command.Delete(ru_fn)
             # GNOME 2.26 (as seen on Ubuntu 9.04) will retain the list
             # in memory if it is simply deleted, so it must be shredded
             # (or at least truncated).
@@ -652,11 +475,11 @@ class System(Cleaner):
 
             def gtk_purge_items():
                 """Purge GTK items"""
-                gtk.RecentManager().purge_items()
+                Gtk.RecentManager().purge_items()
                 yield 0
 
             for pathname in ["~/.recently-used.xbel", "~/.local/share/recently-used.xbel"]:
-                pathname = expanduser(pathname)
+                pathname = os.path.expanduser(pathname)
                 if os.path.lexists(pathname):
                     yield Command.Shred(pathname)
             if HAVE_GTK:
@@ -682,10 +505,10 @@ class System(Cleaner):
 
         # temporary files
         if 'nt' == os.name and 'tmp' == option_id:
-            dirname1 = expandvars(
+            dirname1 = os.path.expandvars(
                 "$USERPROFILE\\Local Settings\\Temp\\")
-            dirname2 = expandvars(r'%temp%')
-            dirname3 = expandvars("%windir%\\temp\\")
+            dirname2 = os.path.expandvars(r'%temp%')
+            dirname3 = os.path.expandvars("%windir%\\temp\\")
             dirnames = []
             if Windows.get_windows_version() >= 6.0:
                 # Windows Vista or later
@@ -704,7 +527,7 @@ class System(Cleaner):
 
         # trash
         if 'posix' == os.name and 'trash' == option_id:
-            dirname = expanduser("~/.Trash")
+            dirname = os.path.expanduser("~/.Trash")
             for filename in children_in_directory(dirname, False):
                 yield Command.Delete(filename)
             # fixme http://www.ramendik.ru/docs/trashspec.html
@@ -712,13 +535,13 @@ class System(Cleaner):
             # ~/.local/share/Trash
             # * GNOME 2.22, Fedora 9
             # * KDE 4.1.3, Ubuntu 8.10
-            dirname = expanduser("~/.local/share/Trash/files")
+            dirname = os.path.expanduser("~/.local/share/Trash/files")
             for filename in children_in_directory(dirname, True):
                 yield Command.Delete(filename)
-            dirname = expanduser("~/.local/share/Trash/info")
+            dirname = os.path.expanduser("~/.local/share/Trash/info")
             for filename in children_in_directory(dirname, True):
                 yield Command.Delete(filename)
-            dirname = expanduser("~/.local/share/Trash/expunged")
+            dirname = os.path.expanduser("~/.local/share/Trash/expunged")
             # desrt@irc.gimpnet.org tells me that the trash
             # backend puts files in here temporary, but in some situations
             # the files are stuck.
@@ -728,10 +551,9 @@ class System(Cleaner):
         # clipboard
         if HAVE_GTK and 'clipboard' == option_id:
             def clear_clipboard():
-                gtk.gdk.threads_enter()
-                clipboard = gtk.clipboard_get()
-                clipboard.set_text("")
-                gtk.gdk.threads_leave()
+                clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+                clipboard.set_text(' ',1)
+                clipboard.clear()
                 return 0
             yield Command.Function(None, clear_clipboard, _('Clipboard'))
 
@@ -761,7 +583,7 @@ class System(Cleaner):
 
         # prefetch
         if 'nt' == os.name and 'prefetch' == option_id:
-            for path in glob.iglob(expandvars('$windir\\Prefetch\\*.pf')):
+            for path in glob.iglob(os.path.expandvars('$windir\\Prefetch\\*.pf')):
                 yield Command.Delete(path)
 
         # recycle bin
@@ -788,7 +610,8 @@ class System(Cleaner):
                 try:
                     Windows.empty_recycle_bin(None, True)
                 except:
-                    logging.getLogger(__name__).info('error in empty_recycle_bin()', exc_info=True)
+                    logging.getLogger(__name__).info(
+                        'error in empty_recycle_bin()', exc_info=True)
                 yield 0
             # Using the Function Command prevents emptying the recycle bin
             # when in preview mode.
@@ -800,13 +623,8 @@ class System(Cleaner):
             for wu in Windows.delete_updates():
                 yield wu
 
-        # return queued files
-        for filename in files:
-            if os.path.lexists(filename):
-                yield Command.Delete(filename)
-
-    def whitelisted(self, pathname):
-        """Return boolean whether file is whitelisted"""
+    def init_whitelist(self):
+        """Initialize the whitelist only once for performance"""
         regexes = [
             '^/tmp/.X0-lock$',
             '^/tmp/.truecrypt_aux_mnt.*/(control|volume)$',
@@ -821,24 +639,39 @@ class System(Cleaner):
             '^/tmp/orbit-[^/]+/bonobo-activation-server-[a-z0-9-]*ior$',
             '^/tmp/pulse-[^/]+/pid$',
             '^/var/tmp/kdecache-',
-            '^' + expanduser('~/.cache/wallpaper/'),
+            '^' + os.path.expanduser('~/.cache/wallpaper/'),
+            # Flatpak mount point
+            '^' + os.path.expanduser('~/.cache/doc($|/)'),
             # Clean Firefox cache from Firefox cleaner (LP#1295826)
-            '^' + expanduser('~/.cache/mozilla'),
+            '^' + os.path.expanduser('~/.cache/mozilla/'),
             # Clean Google Chrome cache from Google Chrome cleaner (LP#656104)
-            '^' + expanduser('~/.cache/google-chrome'),
-            '^' + expanduser('~/.cache/gnome-control-center/'),
+            '^' + os.path.expanduser('~/.cache/google-chrome/'),
+            '^' + os.path.expanduser('~/.cache/gnome-control-center/'),
+            # Clean Evolution cache from Evolution cleaner (GitHub #249)
+            '^' + os.path.expanduser('~/.cache/evolution/'),
             # iBus Pinyin
             # https://bugs.launchpad.net/bleachbit/+bug/1538919
-            '^' + expanduser('~/.cache/ibus/'),
-            # Linux Bluetooth daemon obexd
-            '^' + expanduser('~/.cache/obexd/')]
+            '^' + os.path.expanduser('~/.cache/ibus/'),
+            # Linux Bluetooth daemon obexd directory is typically empty, so be careful
+            # not to delete the empty directory.
+            '^' + os.path.expanduser('~/.cache/obexd($|/)')]
         for regex in regexes:
-            if re.match(regex, pathname) is not None:
+            self.regexes_compiled.append(re.compile(regex))
+
+    def whitelisted(self, pathname):
+        """Return boolean whether file is whitelisted"""
+        if os.name == 'nt':
+            # Whitelist is specific to POSIX
+            return False
+        if not self.regexes_compiled:
+            self.init_whitelist()
+        for regex in self.regexes_compiled:
+            if regex.match(pathname) is not None:
                 return True
         return False
 
 
-def register_cleaners():
+def register_cleaners(cb_progress=lambda x: None, cb_done=lambda: None):
     """Register all known cleaners: system, CleanerML, and Winapp2"""
     global backends
 
@@ -847,18 +680,25 @@ def register_cleaners():
     backends.clear()
 
     # initialize "hard coded" (non-CleanerML) backends
-    backends["firefox"] = Firefox()
     backends["openofficeorg"] = OpenOfficeOrg()
     backends["system"] = System()
 
     # register CleanerML cleaners
     from bleachbit import CleanerML
-    CleanerML.load_cleaners()
+    cb_progress(_('Loading native cleaners.'))
+    for ret in CleanerML.load_cleaners(cb_progress):
+        yield ret
 
     # register Winapp2.ini cleaners
     if 'nt' == os.name:
+        cb_progress(_('Importing cleaners from Winapp2.ini.'))
         from bleachbit import Winapp
-        Winapp.load_cleaners()
+        for ret in Winapp.load_cleaners(cb_progress):
+            yield ret
+
+    cb_done()
+
+    yield False  # end the iteration
 
 
 def create_simple_cleaner(paths):
@@ -874,7 +714,7 @@ def create_simple_cleaner(paths):
 
         def get_commands(self):
             for path in paths:
-                if not isinstance(path, (str, unicode)):
+                if not isinstance(path, (str)):
                     raise RuntimeError(
                         'expected path as string but got %s' % str(path))
                 if not os.path.isabs(path):
