@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -22,13 +22,11 @@
 Create cleaners from CleanerML (markup language)
 """
 
-from __future__ import absolute_import, print_function
-
 import bleachbit
 from bleachbit.Action import ActionProvider
 from bleachbit import _
 from bleachbit.General import boolstr_to_bool, getText
-from bleachbit.FileUtilities import listdir
+from bleachbit.FileUtilities import expand_glob_join, listdir
 from bleachbit import Cleaner
 
 import logging
@@ -39,6 +37,22 @@ import xml.dom.minidom
 logger = logging.getLogger(__name__)
 
 
+def default_vars():
+    """Return default multi-value variables"""
+    ret = {}
+    if not os.name == 'nt':
+        return ret
+    # Expand ProgramFiles to also be ProgramW6432, etc.
+    wowvars = (('ProgramFiles', 'ProgramW6432'),
+               ('CommonProgramFiles', 'CommonProgramW6432'))
+    for v1, v2 in wowvars:
+        # Remove None, if variable is not found.
+        # Make list unique.
+        mylist = list(set([x for x in (os.getenv(v1), os.getenv(v2)) if x]))
+        ret[v1] = mylist
+    return ret
+
+
 class CleanerML:
 
     """Create a cleaner from CleanerML"""
@@ -56,6 +70,7 @@ class CleanerML:
         self.option_name = None
         self.option_description = None
         self.option_warning = None
+        self.vars = default_vars()
         self.xlate_cb = xlate_cb
         if self.xlate_cb is None:
             self.xlate_mode = False
@@ -71,17 +86,34 @@ class CleanerML:
         """Return the created cleaner"""
         return self.cleaner
 
-    def os_match(self, os_str):
-        """Return boolean whether operating system matches"""
+    def os_match(self, os_str, platform=sys.platform):
+        """Return boolean whether operating system matches
+
+        Keyword arguments:
+        os_str -- the required operating system as written in XML
+        platform -- used only for unit tests
+        """
         # If blank or if in .pot-creation-mode, return true.
         if len(os_str) == 0 or self.xlate_mode:
             return True
         # Otherwise, check platform.
-        if os_str == 'linux' and sys.platform.startswith('linux'):
-            return True
-        if os_str == 'windows' and sys.platform == 'win32':
-            return True
-        return False
+        # Define the current operating system.
+        if platform == 'darwin':
+            current_os = ('darwin', 'bsd', 'unix')
+        elif platform.startswith('linux'):
+            current_os = ('linux', 'unix')
+        elif platform.startswith('openbsd'):
+            current_os = ('bsd', 'openbsd', 'unix')
+        elif platform.startswith('netbsd'):
+            current_os = ('bsd', 'netbsd', 'unix')
+        elif platform.startswith('freebsd'):
+            current_os = ('bsd', 'freebsd', 'unix')
+        elif platform == 'win32':
+            current_os = ('windows')
+        else:
+            raise RuntimeError('Unknown operating system: %s ' % sys.platform)
+        # Compare current OS against required OS.
+        return os_str in current_os
 
     def handle_cleaner(self, cleaner):
         """<cleaner> element"""
@@ -92,12 +124,16 @@ class CleanerML:
         description = cleaner.getElementsByTagName('description')
         if description and description[0].parentNode == cleaner:
             self.handle_cleaner_description(description[0])
+        for var in cleaner.getElementsByTagName('var'):
+            self.handle_cleaner_var(var)
         for option in cleaner.getElementsByTagName('option'):
             try:
                 self.handle_cleaner_option(option)
             except:
-                logger.exception('error in handle_cleaner_option() for cleaner id = %s, option XML=%s',
-                                 self.cleaner.id, option.toxml())
+                exc_msg = _(
+                    "Error in handle_cleaner_option() for cleaner id = {cleaner_id}, option XML={option_xml}")
+                logger.exception(exc_msg.format(
+                    cleaner_id=exc_dict, option_xml=option.toxml()))
         self.handle_cleaner_running(cleaner.getElementsByTagName('running'))
         self.handle_localizations(
             cleaner.getElementsByTagName('localizations'))
@@ -119,6 +155,8 @@ class CleanerML:
         """<running> element under <cleaner>"""
         # example: <running type="command">opera</running>
         for running in running_elements:
+            if not self.os_match(running.getAttribute('os')):
+                continue
             detection_type = running.getAttribute('type')
             value = getText(running.childNodes)
             self.cleaner.add_running(detection_type, value)
@@ -166,11 +204,13 @@ class CleanerML:
 
     def handle_cleaner_option_action(self, action_node):
         """<action> element under <option>"""
+        if not self.os_match(action_node.getAttribute('os')):
+            return
         command = action_node.getAttribute('command')
         provider = None
         for actionplugin in ActionProvider.plugins:
             if actionplugin.action_key == command:
-                provider = actionplugin(action_node)
+                provider = actionplugin(action_node, self.vars)
         if provider is None:
             raise RuntimeError("Invalid command '%s'" % command)
         self.cleaner.add_action(self.option_id, provider)
@@ -186,6 +226,34 @@ class CleanerML:
         # Add a dummy action so the file isn't reported as unusable
         self.cleaner.add_action('localization', ActionProvider(None))
 
+    def handle_cleaner_var(self, var):
+        """Handle one <var> element under <cleaner>.
+
+        Example:
+
+        <var name="basepath">
+         <value search="glob">~/.config/f*</value>
+         <value>~/.config/foo</value>
+         <value>%AppData\foo</value>
+         </var>
+        """
+        var_name = var.getAttribute('name')
+        for value_element in var.getElementsByTagName('value'):
+            if not self.os_match(value_element.getAttribute('os')):
+                continue
+            value_str = getText(value_element.childNodes)
+            is_glob = value_element.getAttribute('search') == 'glob'
+            if is_glob:
+                value_list = expand_glob_join(value_str, '')
+            else:
+                value_list = [value_str, ]
+            if var_name in self.vars:
+                # append
+                self.vars[var_name] = value_list + self.vars[var_name]
+            else:
+                # initialize
+                self.vars[var_name] = value_list
+
 
 def list_cleanerml_files(local_only=False):
     """List CleanerML files"""
@@ -201,29 +269,47 @@ def list_cleanerml_files(local_only=False):
         import stat
         st = os.stat(pathname)
         if sys.platform != 'win32' and stat.S_IMODE(st[stat.ST_MODE]) & 2:
-            logger.warning("ignoring cleaner because it is world writable: %s", pathname)
+            # TRANSLATORS: When BleachBit detects the file permissions are
+            # insecure, it will not load the cleaner as if it did not exist.
+            logger.warning(
+                _("Ignoring cleaner because it is world writable: %s"), pathname)
             continue
         yield pathname
 
 
-def load_cleaners():
+def load_cleaners(cb_progress=lambda x: None):
     """Scan for CleanerML and load them"""
-    for pathname in list_cleanerml_files():
+    cleanerml_files = list(list_cleanerml_files())
+    cleanerml_files.sort()
+    if not cleanerml_files:
+        logger.debug('No CleanerML files to load.')
+        return
+    total_files = len(cleanerml_files)
+    cb_progress(0.0)
+    files_done = 0
+    for pathname in cleanerml_files:
         try:
             xmlcleaner = CleanerML(pathname)
         except:
-            logger.exception('error reading cleaner: %s', pathname)
+            logger.exception(_("Error reading cleaner: %s"), pathname)
             continue
         cleaner = xmlcleaner.get_cleaner()
         if cleaner.is_usable():
             Cleaner.backends[cleaner.id] = cleaner
         else:
-            logger.debug('cleaner is not usable on this OS because it has no actions: %s', pathname)
+            logger.debug(
+                # TRANSLATORS: An action is something like cleaning a specific file.
+                # "Not usable" means the whole cleaner will be ignored.
+                # The substituted variable is a pathname.
+                _("Cleaner is not usable on this OS because it has no actions: %s"), pathname)
+        files_done += 1
+        cb_progress(1.0 * files_done / total_files)
+        yield True
 
 
 def pot_fragment(msgid, pathname, translators=None):
     """Create a string fragment for generating .pot files"""
-    msgid = msgid.replace('"', '\\"') # escape quotation mark
+    msgid = msgid.replace('"', '\\"')  # escape quotation mark
     if translators:
         translators = "#. %s\n" % translators
     else:
@@ -250,7 +336,7 @@ def create_pot():
                       lambda newstr, translators=None:
                       strings.append([newstr, translators]))
         except:
-            logger.exception('error reading: %s', pathname)
+            logger.exception(_("Error reading cleaner: %s"), pathname)
             continue
         for (string, translators) in strings:
             f.write(pot_fragment(string, pathname, translators))
