@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -21,14 +21,11 @@
 General code
 """
 
-from __future__ import absolute_import, print_function
-
-import bleachbit
-
 import logging
 import os
 import sys
-import traceback
+
+import bleachbit
 
 logger = logging.getLogger(__name__)
 
@@ -80,7 +77,7 @@ def chownself(path):
     try:
         os.chown(path, uid, -1)
     except:
-        traceback.print_exc()
+        logger.exception('Error in chown() under chownself()')
 
 
 def getrealuid():
@@ -110,7 +107,7 @@ def getrealuid():
 def makedirs(path):
     """Make directory recursively considering sudo permissions.
     'Path' should not end in a delimiter."""
-    logger.debug('makedirs(%s)', path.encode(bleachbit.FSE))
+    logger.debug('makedirs(%s)', path)
     if os.path.lexists(path):
         return
     parentdir = os.path.split(path)[0]
@@ -121,20 +118,23 @@ def makedirs(path):
         chownself(path)
 
 
-def run_external(args, stdout=False, env=None, clean_env=True):
+def run_external(args, stdout=None, env=None, clean_env=True):
     """Run external command and return (return code, stdout, stderr)"""
     logger.debug('running cmd ' + ' '.join(args))
     import subprocess
-    if not stdout:
+    if stdout is None:
         stdout = subprocess.PIPE
     kwargs = {}
-    if subprocess.mswindows:
+    encoding = bleachbit.stdout_encoding
+    if sys.platform == 'win32':
         # hide the 'DOS box' window
-        import win32process, win32con
+        import win32process
+        import win32con
         stui = subprocess.STARTUPINFO()
         stui.dwFlags = win32process.STARTF_USESHOWWINDOW
         stui.wShowWindow = win32con.SW_HIDE
         kwargs['startupinfo'] = stui
+        encoding='mbcs'
     if not env and clean_env and 'posix' == os.name:
         # Clean environment variables so that that subprocesses use English
         # instead of translated text. This helps when checking for certain
@@ -143,7 +143,7 @@ def run_external(args, stdout=False, env=None, clean_env=True):
         # https://github.com/bleachbit/bleachbit/issues/168
         keep_env = ('PATH', 'HOME', 'LD_LIBRARY_PATH', 'TMPDIR')
         env = dict((key, value)
-                   for key, value in os.environ.iteritems() if key in keep_env)
+                   for key, value in os.environ.items() if key in keep_env)
         env['LANG'] = 'C'
         env['LC_ALL'] = 'C'
     p = subprocess.Popen(args, stdout=stdout,
@@ -155,7 +155,9 @@ def run_external(args, stdout=False, env=None, clean_env=True):
         print(out[0])
         print(out[1])
         raise
-    return p.returncode, out[0], out[1]
+    return (p.returncode,
+            str(out[0], encoding=encoding) if out[0] else '',
+            str(out[1], encoding=encoding) if out[1] else '')
 
 
 def sudo_mode():
