@@ -2,8 +2,8 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2020 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,24 +23,26 @@
 Wipe memory
 """
 
+from bleachbit import FileUtilities
+from bleachbit import General
+from bleachbit import _
 
+import logging
 import os
 import re
 import subprocess
 import sys
-import traceback
 
-import FileUtilities
-import General
+logger = logging.getLogger(__name__)
 
 
 def count_swap_linux():
     """Count the number of swap devices in use"""
-    f = open("/proc/swaps")
     count = 0
-    for line in f:
-        if line[0] == '/':
-            count += 1
+    with open("/proc/swaps") as f:
+        for line in f:
+            if line[0] == '/':
+                count += 1
     return count
 
 
@@ -49,10 +51,11 @@ def get_proc_swaps():
     # Usually 'swapon -s' is identical to '/proc/swaps'
     # Here is one exception:
     # https://bugs.launchpad.net/ubuntu/+source/bleachbit/+bug/1092792
-    (rc, stdout, _) = General.run_external(['swapon', '-s'])
+    (rc, stdout, _stderr) = General.run_external(['swapon', '-s'])
     if 0 == rc:
         return stdout
-    print 'debug: "swapoff -s" failed so falling back to /proc/swaps'
+    logger.debug(
+        _("The command 'swapoff -s' failed, so falling back to /proc/swaps for swap information."))
     return open("/proc/swaps").read()
 
 
@@ -61,7 +64,7 @@ def parse_swapoff(swapoff):
     # English is 'swapoff on /dev/sda5' but German is 'swapoff für ...'
     # Example output in English with LVM and hyphen: 'swapoff on /dev/mapper/lubuntu-swap_1'
     # This matches swap devices and swap files
-    ret = re.search('^swapoff (\w* )?(/[\w/\.-]+)$', swapoff)
+    ret = re.search('^swapoff (\w* )?(/[\w/.-]+)$', swapoff)
     if not ret:
         # no matches
         return None
@@ -72,7 +75,7 @@ def disable_swap_linux():
     """Disable Linux swap and return list of devices"""
     if 0 == count_swap_linux():
         return
-    print "debug: disabling swap"
+    logger.debug(_("Disabling swap."))
     args = ["swapoff", "-a", "-v"]
     (rc, stdout, stderr) = General.run_external(args)
     if 0 != rc:
@@ -83,7 +86,7 @@ def disable_swap_linux():
         if '' == line:
             continue
         ret = parse_swapoff(line)
-        if None == ret:
+        if ret is None:
             raise RuntimeError("Unexpected output:\nargs='%(args)s'\nstdout='%(stdout)s'\nstderr='%(stderr)s'"
                                % {'args': str(args), 'stdout': stdout, 'stderr': stderr})
         devices.append(ret)
@@ -92,7 +95,7 @@ def disable_swap_linux():
 
 def enable_swap_linux():
     """Enable Linux swap"""
-    print "debug: re-enabling swap"
+    logger.debug(_("Re-enabling swap."))
     args = ["swapon", "-a"]
     p = subprocess.Popen(args, stderr=subprocess.PIPE)
     p.wait()
@@ -106,22 +109,24 @@ def make_self_oom_target_linux():
     # In Linux 2.6.36 the system changed from oom_adj to oom_score_adj
     path = '/proc/%d/oom_score_adj' % os.getpid()
     if os.path.exists(path):
-        open(path, 'w').write('1000')
+        with open(path, 'w') as f:
+            f.write('1000')
     else:
         path = '/proc/%d/oomadj' % os.getpid()
         if os.path.exists(path):
-            open(path, 'w').write('15')
+            with open(path, 'w') as f:
+                f.write('15')
     # OOM likes nice processes
-    print 'debug: new nice value', os.nice(19)
+    logger.debug(_("Setting nice value %d for this process."), os.nice(19))
     # OOM prefers non-privileged processes
     try:
         uid = General.getrealuid()
         if uid > 0:
-            print "debug: dropping privileges of pid %d to uid %d" % \
-                (os.getpid(), uid)
+            logger.debug(
+                _("Dropping privileges of process ID {pid} to user ID {uid}.").format(pid=os.getpid(), uid=uid))
             os.seteuid(uid)
     except:
-        traceback.print_exc()
+        logger.exception('Error when dropping privileges')
 
 
 def fill_memory_linux():
@@ -131,21 +136,24 @@ def fill_memory_linux():
     if allocbytes < 1024:
         return
     bytes_str = FileUtilities.bytes_to_human(allocbytes)
-    print "info: allocating and wiping %s (%d B) of memory" % (bytes_str, allocbytes)
+    # TRANSLATORS: The variable is a quantity like 5kB
+    logger.info(_("Allocating and wiping %s of memory."),
+                bytes_str)
     try:
         buf = '\x00' * allocbytes
     except MemoryError:
         pass
     else:
         fill_memory_linux()
-        print "debug: freeing %s of memory" % bytes_str
+        # TRANSLATORS: The variable is a quantity like 5kB
+        logger.debug(_("Freeing %s of memory."), bytes_str)
         del buf
     report_free()
 
 
 def get_swap_size_linux(device, proc_swaps=None):
     """Return the size of the partition in bytes"""
-    if None == proc_swaps:
+    if proc_swaps is None:
         proc_swaps = get_proc_swaps()
     line = proc_swaps.split('\n')[0]
     if not re.search('Filename\s+Type\s+Size', line):
@@ -162,26 +170,47 @@ def get_swap_uuid(device):
     """Find the UUID for the swap device"""
     uuid = None
     args = ['blkid', device, '-s', 'UUID']
-    (_, stdout, _) = General.run_external(args)
+    (_rc, stdout, _stderr) = General.run_external(args)
     for line in stdout.split('\n'):
         # example: /dev/sda5: UUID="ee0e85f6-6e5c-42b9-902f-776531938bbf"
         ret = re.search("^%s: UUID=\"([a-z0-9-]+)\"" % device, line)
-        if None != ret:
+        if ret is not None:
             uuid = ret.group(1)
-    print "debug: uuid(%s)='%s'" % (device, uuid)
+    logger.debug(_("Found UUID for swap file {device} is {uuid}.").format(
+        device=device, uuid=uuid))
     return uuid
 
 
+def physical_free_darwin(run_vmstat=None):
+    def parse_line(k, v):
+        return k, int(v.strip(" ."))
+
+    def get_page_size(line):
+        m = re.match(
+            r"Mach Virtual Memory Statistics: \(page size of (\d+) bytes\)",
+            line)
+        if m is None:
+            raise RuntimeError("Can't parse vm_stat output")
+        return int(m.groups()[0])
+    if run_vmstat is None:
+        def run_vmstat():
+            return subprocess.check_output(["vm_stat"])
+    output = iter(run_vmstat().split("\n"))
+    page_size = get_page_size(next(output))
+    vm_stat = dict(parse_line(*l.split(":")) for l in output if l != "")
+    return vm_stat["Pages free"] * page_size
+
+
 def physical_free_linux():
     """Return the physical free memory on Linux"""
-    f = open("/proc/meminfo")
     free_bytes = 0
-    for line in f:
-        line = line.replace("\n", "")
-        ret = re.search('(MemFree|Cached):[ ]*([0-9]*) kB', line)
-        if None != ret:
-            kb = int(ret.group(2))
-            free_bytes += kb * 1024
+    with open("/proc/meminfo") as f:
+        for line in f:
+            line = line.replace("\n", "")
+            ret = re.search('(MemFree|Cached):[ ]*([0-9]*) kB', line)
+            if ret is not None:
+                kb = int(ret.group(2))
+                free_bytes += kb * 1024
     if free_bytes > 0:
         return free_bytes
     else:
@@ -214,7 +243,6 @@ def physical_free_windows():
         return x
 
     z = GlobalMemoryStatusEx()
-    print z
     return z.ullAvailPhys
 
 
@@ -223,6 +251,8 @@ def physical_free():
         return physical_free_linux()
     elif 'win32' == sys.platform:
         return physical_free_windows()
+    elif 'darwin' == sys.platform:
+        return physical_free_darwin()
     else:
         raise RuntimeError('unsupported platform for physical_free()')
 
@@ -231,31 +261,40 @@ def report_free():
     """Report free memory"""
     bytes_free = physical_free()
     bytes_str = FileUtilities.bytes_to_human(bytes_free)
-    print "debug: physical free: %s (%d B)" % \
-        (bytes_str, bytes_free)
+    # TRANSLATORS: The variable is a quantity like 5kB
+    logger.debug(_("Physical free memory is %s."),
+                 bytes_str)
 
 
 def wipe_swap_linux(devices, proc_swaps):
-    """Shred the Linux swap file and then reinitilize it"""
-    if None == devices:
+    """Shred the Linux swap file and then reinitialize it"""
+    if devices is None:
         return
     if 0 < count_swap_linux():
         raise RuntimeError('Cannot wipe swap while it is in use')
     for device in devices:
-        print "info: wiping swap device '%s'" % device
-        if get_swap_size_linux(device, proc_swaps) > 8 * 1024 ** 3:
+        # if '/cryptswap' in device:
+        #    logger.info('Skipping encrypted swap device %s.', device)
+        #    continue
+        # TRANSLATORS: The variable is a device like /dev/sda2
+        logger.info(_("Wiping the swap device %s."), device)
+        safety_limit_bytes = 29 * 1024 ** 3  # 29 gibibytes
+        actual_size_bytes = get_swap_size_linux(device, proc_swaps)
+        if actual_size_bytes > safety_limit_bytes:
             raise RuntimeError(
-                'swap device %s is larger than expected' % device)
+                'swap device %s is larger (%d) than expected (%d)' %
+                (device, actual_size_bytes, safety_limit_bytes))
         uuid = get_swap_uuid(device)
         # wipe
         FileUtilities.wipe_contents(device, truncate=False)
         # reinitialize
-        print "debug: reinitializing swap device %s" % device
+        # TRANSLATORS: The variable is a device like /dev/sda2
+        logger.debug(_("Reinitializing the swap device %s."), device)
         args = ['mkswap', device]
         if uuid:
             args.append("-U")
             args.append(uuid)
-        (rc, _, stderr) = General.run_external(args)
+        (rc, _stdout, stderr) = General.run_external(args)
         if 0 != rc:
             raise RuntimeError(stderr.replace("\n", ""))
 
@@ -266,19 +305,22 @@ def wipe_memory():
     proc_swaps = get_proc_swaps()
     devices = disable_swap_linux()
     yield True  # process GTK+ idle loop
-    print 'debug: detected swap devices:', devices
+    # TRANSLATORS: The variable is a device like /dev/sda2
+    logger.debug(_("Detected these swap devices: %s"), str(devices))
     wipe_swap_linux(devices, proc_swaps)
     yield True
     child_pid = os.fork()
     if 0 == child_pid:
         make_self_oom_target_linux()
         fill_memory_linux()
-        sys.exit(0)
+        os._exit(0)
     else:
-        print 'debug: wipe_memory() pid %d waiting for child pid %d' % \
-            (os.getpid(), child_pid)
+        # TRANSLATORS: This is a debugging message that the parent process is waiting for the child process.
+        logger.debug(_("The function wipe_memory() with process ID {pid} is waiting for child process ID {cid}.").format(
+                     pid=os.getpid(), cid=child_pid))
         rc = os.waitpid(child_pid, 0)[1]
         if 0 != rc:
-            print 'warning: child process returned code %d' % rc
+            logger.warning(
+                _("The child memory-wiping process returned code %d."), rc)
     enable_swap_linux()
     yield 0  # how much disk space was recovered
