@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -22,15 +22,14 @@
 Store and retrieve user preferences
 """
 
-from __future__ import absolute_import, print_function
-
 import bleachbit
 from bleachbit import General
+from bleachbit import _
+from bleachbit.Log import set_root_log_level
 
 import logging
 import os
 import re
-import traceback
 
 logger = logging.getLogger(__name__)
 
@@ -38,10 +37,22 @@ if 'nt' == os.name:
     from win32file import GetLongPathName
 
 
-boolean_keys = ['auto_hide', 'auto_start', 'check_beta',
-                'check_online_updates', 'first_start', 'shred', 'exit_done', 'delete_confirmation', 'units_iec']
+boolean_keys = ['auto_hide',
+                'check_beta',
+                'check_online_updates',
+                'dark_mode',
+                'delete_confirmation',
+                'debug',
+                'exit_done',
+                'first_start',
+                'shred',
+                'units_iec',
+                'window_maximized',
+                'window_fullscreen']
 if 'nt' == os.name:
     boolean_keys.append('update_winapp2')
+    boolean_keys.append('win10_theme')
+int_keys = ['window_x', 'window_y', 'window_width', 'window_height', ]
 
 
 def path_to_option(pathname):
@@ -56,16 +67,31 @@ def path_to_option(pathname):
         pathname = pathname[0] + pathname[2:]
     return pathname
 
+
+def init_configuration():
+    """Initialize an empty configuration, if necessary"""
+    if os.path.lexists(bleachbit.options_file):
+        logger.debug('Deleting configuration: %s ' % bleachbit.options_file)
+        os.remove(bleachbit.options_file)
+    with open(bleachbit.options_file, 'w', encoding='utf-8-sig') as f_ini:
+        f_ini.write('[bleachbit]\n')
+        if os.name == 'nt' and bleachbit.portable_mode:
+            f_ini.write('[Portable]\n')
+    for section in options.config.sections():
+        options.config.remove_section(section)
+    options.restore()
+
+
 class Options:
 
     """Store and retrieve user preferences"""
 
     def __init__(self):
         self.purged = False
-        self.config = bleachbit.SafeConfigParser()
+        self.config = bleachbit.RawConfigParser()
         self.config.optionxform = str  # make keys case sensitive for hashpath purging
-        self.config._boolean_states['t'] = True
-        self.config._boolean_states['f'] = False
+        self.config.BOOLEAN_STATES['t'] = True
+        self.config.BOOLEAN_STATES['f'] = False
         self.restore()
 
     def __flush(self):
@@ -75,16 +101,16 @@ class Options:
         if not os.path.exists(bleachbit.options_dir):
             General.makedirs(bleachbit.options_dir)
         mkfile = not os.path.exists(bleachbit.options_file)
-        _file = open(bleachbit.options_file, 'wb')
-        try:
-            self.config.write(_file)
-        except IOError as e:
-            print(e)
-            from errno import ENOSPC
-            if e.errno == ENOSPC:
-                logger.error("disk is full writing configuration '%s'", bleachbit.options_file)
-            else:
-                raise
+        with open(bleachbit.options_file, 'w', encoding='utf-8-sig') as _file:
+            try:
+                self.config.write(_file)
+            except IOError as e:
+                from errno import ENOSPC
+                if e.errno == ENOSPC:
+                    logger.error(
+                        _("Disk was full when writing configuration to file %s"), bleachbit.options_file)
+                else:
+                    raise
         if mkfile and General.sudo_mode():
             General.chownself(bleachbit.options_file)
 
@@ -99,14 +125,14 @@ class Options:
                 # restore colon lost because ConfigParser treats colon special
                 # in keys
                 pathname = pathname[0] + ':' + pathname[1:]
-                pathname = pathname.decode('utf-8')
             exists = False
             try:
                 exists = os.path.lexists(pathname)
             except:
                 # this deals with corrupt keys
                 # https://www.bleachbit.org/forum/bleachbit-wont-launch-error-startup
-                logger.error('error checking whether path exists: %s ', pathname)
+                logger.error(
+                    _("Error checking whether path exists: %s"), pathname)
             if not exists:
                 # the file does not on exist, so forget it
                 self.config.remove_option('hashpath', option)
@@ -116,6 +142,10 @@ class Options:
         if not self.config.has_option('bleachbit', key):
             self.set(key, value)
 
+    def has_option(self, option, section='bleachbit'):
+        """Check if option is set"""
+        return self.config.has_option(section, option)
+
     def get(self, option, section='bleachbit'):
         """Retrieve a general option"""
         if not 'nt' == os.name and 'update_winapp2' == option:
@@ -123,8 +153,14 @@ class Options:
         if section == 'hashpath' and option[1] == ':':
             option = option[0] + option[2:]
         if option in boolean_keys:
+            from bleachbit.Log import is_debugging_enabled_via_cli
+            if section == 'bleachbit' and option == 'debug' and is_debugging_enabled_via_cli():
+                # command line overrides store configuration
+                return True
             return self.config.getboolean(section, option)
-        return self.config.get(section, option.encode('utf-8'))
+        elif option in int_keys:
+            return self.config.getint(section, option)
+        return self.config.get(section, option)
 
     def get_hashpath(self, pathname):
         """Recall the hash for a file"""
@@ -188,15 +224,33 @@ class Options:
             return self.config.getboolean('tree', option)
         except:
             # in case of corrupt configuration (Launchpad #799130)
-            traceback.print_exc()
+            logger.exception('Error in get_tree()')
             return False
 
+    def is_corrupt(self):
+        """Perform a self-check for corruption of the configuration"""
+        # no boolean key must raise an exception
+        for boolean_key in boolean_keys:
+            try:
+                if self.config.has_option('bleachbit', boolean_key):
+                    self.config.getboolean('bleachbit', boolean_key)
+            except ValueError:
+                return True
+        # no int key must raise an exception
+        for int_key in int_keys:
+            try:
+                if self.config.has_option('bleachbit', int_key):
+                    self.config.getint('bleachbit', int_key)
+            except ValueError:
+                return True
+        return False
+
     def restore(self):
         """Restore saved options from disk"""
         try:
-            self.config.read(bleachbit.options_file)
+            self.config.read(bleachbit.options_file, encoding='utf-8-sig')
         except:
-            traceback.print_exc()
+            logger.exception("Error reading application's configuration")
         if not self.config.has_section("bleachbit"):
             self.config.add_section("bleachbit")
         if not self.config.has_section("hashpath"):
@@ -206,21 +260,25 @@ class Options:
             try:
                 self.set_list('shred_drives', guess_overwrite_paths())
             except:
-                traceback.print_exc()
-                logger.error('error setting default shred drives')
+                logger.exception(
+                    _("Error when setting the default drives to shred."))
 
         # set defaults
         self.__set_default("auto_hide", True)
-        self.__set_default("auto_start", False)
         self.__set_default("check_beta", False)
         self.__set_default("check_online_updates", True)
-        self.__set_default("shred", False)
-        self.__set_default("exit_done", False)
+        self.__set_default("dark_mode", True)
         self.__set_default("delete_confirmation", True)
+        self.__set_default("debug", False)
+        self.__set_default("exit_done", False)
+        self.__set_default("shred", False)
         self.__set_default("units_iec", False)
+        self.__set_default("window_fullscreen", False)
+        self.__set_default("window_maximized", False)
 
         if 'nt' == os.name:
             self.__set_default("update_winapp2", False)
+            self.__set_default("win10_theme", False)
 
         if not self.config.has_section('preserve_languages'):
             lang = bleachbit.user_locale
@@ -228,7 +286,7 @@ class Options:
             if -1 != pos:
                 lang = lang[0: pos]
             for _lang in set([lang, 'en']):
-                logger.info("automatically preserving language '%s'", lang)
+                logger.info(_("Automatically preserving language %s."), lang)
                 self.set_language(_lang, True)
 
         # BleachBit upgrade or first start ever
@@ -241,10 +299,13 @@ class Options:
 
     def set(self, key, value, section='bleachbit', commit=True):
         """Set a general option"""
-        self.config.set(section, key.encode('utf-8'), str(value))
+        self.config.set(section, key, str(value))
         if commit:
             self.__flush()
 
+    def commit(self):
+        self.__flush()
+
     def set_hashpath(self, pathname, hashvalue):
         """Remember the hash of a path"""
         self.set(path_to_option(pathname), hashvalue, 'hashpath')
@@ -316,3 +377,6 @@ class Options:
 
 
 options = Options()
+
+# Now that the configuration is loaded, honor the debug preference there.
+set_root_log_level()
