@@ -2,7 +2,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,22 +23,19 @@
 Import Winapp2.ini files
 """
 
-from __future__ import absolute_import, print_function
-
-import bleachbit
-from bleachbit import Cleaner, Windows
-from bleachbit.Action import Delete, Winreg
-from bleachbit import _, FSE, expandvars
-
 import logging
 import os
 import glob
 import re
 from xml.dom.minidom import parseString
 
+import bleachbit
+from bleachbit import Cleaner, Windows
+from bleachbit.Action import Delete, Winreg
+from bleachbit import _
+
 logger = logging.getLogger(__name__)
 
-MAX_DETECT = 50
 
 # TRANSLATORS: This is cleaner name for cleaners imported from winapp2.ini
 langsecref_map = {'3021': ('winapp2_applications', _('Applications')),
@@ -82,17 +79,18 @@ def detectos(required_ver, mock=False):
     current operating system, or the mock version, if given."""
     # Do not compare as string because Windows 10 (build 10.0) comes after
     # Windows 8.1 (build 6.3).
-    assert(isinstance(required_ver, (str, unicode)))
+    assert isinstance(required_ver, str)
     current_os = (mock if mock else Windows.parse_windows_build())
     required_ver = required_ver.strip()
-    if required_ver.startswith('|'):
-        # This is the maximum version
-        # For example, |5.1 means Windows XP (5.1) but not Vista (6.0)
-        return current_os <= Windows.parse_windows_build(required_ver[1:])
-    elif required_ver.endswith('|'):
-        # This is the minimum version
-        # For example, 6.1| means Windows 7 or later
-        return current_os >= Windows.parse_windows_build(required_ver[:-1])
+    if '|' in required_ver:
+        # Format of min|max
+        req_min = required_ver.split('|')[0]
+        req_max = required_ver.split('|')[1]
+        if req_min and current_os < Windows.parse_windows_build(req_min):
+            return False
+        if req_max and current_os > Windows.parse_windows_build(req_max):
+            return False
+        return True
     else:
         # Exact version
         return Windows.parse_windows_build(required_ver) == current_os
@@ -101,7 +99,7 @@ def detectos(required_ver, mock=False):
 def winapp_expand_vars(pathname):
     """Expand environment variables using special Winapp2.ini rules"""
     # This is the regular expansion
-    expand1 = expandvars(pathname)
+    expand1 = os.path.expandvars(pathname)
     # Winapp2.ini expands %ProgramFiles% to %ProgramW6432%, etc.
     subs = (('ProgramFiles', 'ProgramW6432'),
             ('CommonProgramFiles', 'CommonProgramW6432'))
@@ -109,7 +107,7 @@ def winapp_expand_vars(pathname):
         pattern = re.compile('%{}%'.format(sub_orig), flags=re.IGNORECASE)
         if pattern.match(pathname):
             expand2 = pattern.sub('%{}%'.format(sub_repl), pathname)
-            return expand1, expandvars(expand2)
+            return expand1, os.path.expandvars(expand2)
     return expand1,
 
 
@@ -121,22 +119,38 @@ def detect_file(pathname):
     return False
 
 
+def special_detect(code):
+    """Check whether the SpecialDetect== software exists"""
+    # The last two are used only for testing
+    sd_keys = {'DET_CHROME': r'HKCU\Software\Google\Chrome',
+               'DET_MOZILLA': r'HKCU\Software\Mozilla\Firefox',
+               'DET_OPERA': r'HKCU\Software\Opera Software',
+               'DET_THUNDERBIRD': r'HKLM\SOFTWARE\Clients\Mail\Mozilla Thunderbird',
+               'DET_WINDOWS': r'HKCU\Software\Microsoft',
+               'DET_SPACE_QUEST': r'HKCU\Software\Sierra Games\Space Quest'}
+    if code in sd_keys:
+        return Windows.detect_registry_key(sd_keys[code])
+    else:
+        logger.error('Unknown SpecialDetect=%s', code)
+    return False
+
+
 def fnmatch_translate(pattern):
     """Same as the original without the end"""
     import fnmatch
-    r = fnmatch.translate(pattern)
-    if r.endswith('$'):
-        return r[:-1]
-    if r.endswith(r'\Z(?ms)'):
-        return r[:-7]
-    return r
+    ret = fnmatch.translate(pattern)
+    if ret.endswith('$'):
+        return ret[:-1]
+    if ret.endswith(r'\Z(?ms)'):
+        return ret[:-7]
+    return ret
 
 
 class Winapp:
 
     """Create cleaners from a Winapp2.ini-style file"""
 
-    def __init__(self, pathname):
+    def __init__(self, pathname, cb_progress=lambda x: None):
         """Create cleaners from a Winapp2.ini-style file"""
 
         self.cleaners = {}
@@ -146,12 +160,20 @@ class Winapp:
         self.errors = 0
         self.parser = bleachbit.RawConfigParser()
         self.parser.read(pathname)
+        self.re_detect = re.compile(r'^detect(\d+)?$')
+        self.re_detectfile = re.compile(r'^detectfile(\d+)?$')
+        self.re_excludekey = re.compile(r'^excludekey\d+$')
+        section_total_count = len(self.parser.sections())
+        section_done_count = 0
         for section in self.parser.sections():
             try:
                 self.handle_section(section)
             except Exception:
                 self.errors += 1
                 logger.exception('parsing error in section %s', section)
+            else:
+                section_done_count += 1
+                cb_progress(1.0*section_done_count/section_total_count)
 
     def add_section(self, cleaner_id, name):
         """Add a section (cleaners)"""
@@ -160,6 +182,9 @@ class Winapp:
         self.cleaners[cleaner_id].id = cleaner_id
         self.cleaners[cleaner_id].name = name
         self.cleaners[cleaner_id].description = _('Imported from winapp2.ini')
+        # The detect() function in this module effectively does what
+        # auto_hide() does, so this avoids redundant, slow processing.
+        self.cleaners[cleaner_id].auto_hide = lambda: False
 
     def section_to_cleanerid(self, langsecref):
         """Given a langsecref (or section name), find the internal
@@ -175,7 +200,7 @@ class Winapp:
         return cleanerid
 
     def excludekey_to_nwholeregex(self, excludekey):
-        """Translate one ExcludeKey to CleanerML nwholeregex
+        r"""Translate one ExcludeKey to CleanerML nwholeregex
 
         Supported examples
         FILE=%LocalAppData%\BleachBit\BleachBit.ini
@@ -187,18 +212,18 @@ class Winapp:
         """
         parts = excludekey.split('|')
         parts[0] = parts[0].upper()
-        if 'REG' == parts[0]:
+        if parts[0] == 'REG':
             raise NotImplementedError('REG not supported in ExcludeKey')
 
         # the last part contains the filename(s)
         files = None
         files_regex = ''
-        if 3 == len(parts):
+        if len(parts) == 3:
             files = parts[2].split(';')
-            if 1 == len(files):
+            if len(files) == 1:
                 # one file pattern like *.* or *.log
                 files_regex = fnmatch_translate(files[0])
-                if '*.*' == files_regex:
+                if files_regex == '*.*':
                     files = None
             elif len(files) > 1:
                 # multiple file patterns like *.log;*.bak
@@ -220,61 +245,66 @@ class Winapp:
                     fnmatch_translate(expanded), files_regex)
             regexes.append(regex)
 
-        if 1 == len(regexes):
+        if len(regexes) == 1:
             return regexes[0]
         else:
             return '(%s)' % '|'.join(regexes)
 
+    def detect(self, section):
+        """Check whether to show the section
+
+        The logic:
+        If the DetectOS does not match, the section is inactive.
+        If any Detect or DetectFile matches, the section is active.
+        If neither Detect or DetectFile was given, the section is active.
+        Otherwise, the section is inactive.
+        """
+        if self.parser.has_option(section, 'detectos'):
+            required_ver = self.parser.get(section, 'detectos')
+            if not detectos(required_ver):
+                return False
+        any_detect_option = False
+        if self.parser.has_option(section, 'specialdetect'):
+            any_detect_option = True
+            sd_code = self.parser.get(section, 'specialdetect')
+            if special_detect(sd_code):
+                return True
+        for option in self.parser.options(section):
+            if re.match(self.re_detect, option):
+                # Detect= checks for a registry key
+                any_detect_option = True
+                key = self.parser.get(section, option)
+                if Windows.detect_registry_key(key):
+                    return True
+            elif re.match(self.re_detectfile, option):
+                # DetectFile= checks for a file
+                any_detect_option = True
+                key = self.parser.get(section, option)
+                if detect_file(key):
+                    return True
+        return not any_detect_option
+
     def handle_section(self, section):
         """Parse a section"""
-        # if simple detection fails then discard the section
-        if self.parser.has_option(section, 'detect'):
-            key = self.parser.get(section, 'detect').decode(FSE)
-            if not Windows.detect_registry_key(key):
-                return
-        if self.parser.has_option(section, 'detectfile'):
-            if not detect_file(self.parser.get(section, 'detectfile').decode(FSE)):
-                return
-        if self.parser.has_option(section, 'detectos'):
-            required_ver = self.parser.get(section, 'detectos').decode(FSE)
-            if not detectos(required_ver):
-                return
-        # in case of multiple detection, discard if none match
-        if self.parser.has_option(section, 'detectfile1'):
-            matches = 0
-            for n in range(1, MAX_DETECT):
-                option_id = 'detectfile%d' % n
-                if self.parser.has_option(section, option_id):
-                    if detect_file(self.parser.get(section, option_id).decode(FSE)):
-                        matches += 1
-            if 0 == matches:
-                return
-        if self.parser.has_option(section, 'detect1'):
-            matches = 0
-            for n in range(1, MAX_DETECT):
-                option_id = 'detect%d' % n
-                if self.parser.has_option(section, option_id):
-                    if Windows.detect_registry_key(self.parser.get(section, option_id).decode(FSE)):
-                        matches += 1
-            if 0 == matches:
-                return
+        # check whether the section is active (i.e., whether it will be shown)
+        if not self.detect(section):
+            return
         # excludekeys ignores a file, path, or registry key
         excludekeys = []
-        if self.parser.has_option(section, 'excludekey1'):
-            for n in range(1, MAX_DETECT):
-                option_id = 'excludekey%d' % n
-                if self.parser.has_option(section, option_id):
-                    excludekeys.append(
-                        self.excludekey_to_nwholeregex(self.parser.get(section, option_id).decode(FSE)))
+        for option in self.parser.options(section):
+            if re.match(self.re_excludekey, option):
+                excludekeys.append(
+                    self.excludekey_to_nwholeregex(self.parser.get(section, option)))
         # there are two ways to specify sections: langsecref= and section=
         if self.parser.has_option(section, 'langsecref'):
             # verify the langsecref number is known
             # langsecref_num is 3021, games, etc.
-            langsecref_num = self.parser.get(section, 'langsecref').decode(FSE)
+            langsecref_num = self.parser.get(section, 'langsecref')
         elif self.parser.has_option(section, 'section'):
-            langsecref_num = self.parser.get(section, 'section').decode(FSE)
+            langsecref_num = self.parser.get(section, 'section')
         else:
-            logger.error('neither option LangSecRef nor Section found in section %s', section)
+            logger.error(
+                'neither option LangSecRef nor Section found in section %s', section)
             return
         # find the BleachBit internal cleaner ID
         lid = self.section_to_cleanerid(langsecref_num)
@@ -287,13 +317,15 @@ class Winapp:
                 self.handle_regkey(lid, section, option)
             elif option == 'warning':
                 self.cleaners[lid].set_warning(
-                    section2option(section), self.parser.get(section, 'warning').decode(FSE))
-            elif option in ('default', 'detectfile', 'detect', 'langsecref', 'section') \
-                or ['detect%d' % x for x in range(1, MAX_DETECT)] \
-                    or ['detectfile%d' % x for x in range(1, MAX_DETECT)]:
+                    section2option(section), self.parser.get(section, 'warning'))
+            elif option in ('default', 'langsecref', 'section', 'detectos', 'specialdetect') \
+                    or re.match(self.re_detect, option) \
+                    or re.match(self.re_detectfile, option) \
+                    or re.match(self.re_excludekey, option):
                 pass
             else:
-                logger.warning('unknown option %s in section %s', option, section)
+                logger.warning(
+                    'unknown option %s in section %s', option, section)
                 return
 
     def __make_file_provider(self, dirname, filename, recurse, removeself, excludekeys):
@@ -304,7 +336,7 @@ class Winapp:
             path = dirname
             if filename.startswith('*.'):
                 filename = filename.replace('*.', '.')
-            if '.*' == filename:
+            if filename == '.*':
                 if removeself:
                     search = 'walk.all'
             else:
@@ -313,7 +345,7 @@ class Winapp:
         else:
             search = 'glob'
             path = os.path.join(dirname, filename)
-            if -1 == path.find('*'):
+            if path.find('*') == -1:
                 search = 'file'
         excludekeysxml = ''
         if excludekeys:
@@ -323,12 +355,12 @@ class Winapp:
             else:
                 # just one
                 exclude_str = excludekeys[0]
-            excludekeysxml = 'nwholeregex="%s"' % exclude_str
-        action_str = u'<option command="delete" search="%s" path="%s" %s %s/>' % \
+            excludekeysxml = 'nwholeregex="%s"' % xml_escape(exclude_str)
+        action_str = '<option command="delete" search="%s" path="%s" %s %s/>' % \
                      (search, xml_escape(path), regex, excludekeysxml)
         yield Delete(parseString(action_str).childNodes[0])
         if removeself:
-            action_str = u'<option command="delete" search="file" path="%s"/>' % \
+            action_str = '<option command="delete" search="file" path="%s"/>' % \
                          (xml_escape(dirname))
             yield Delete(parseString(action_str).childNodes[0])
 
@@ -336,7 +368,8 @@ class Winapp:
         """Parse a FileKey# option.
 
         Section is [Application Name] and option is the FileKey#"""
-        elements = self.parser.get(ini_section, ini_option).decode(FSE).strip().split('|')
+        elements = self.parser.get(
+            ini_section, ini_option).strip().split('|')
         dirnames = winapp_expand_vars(elements.pop(0))
         filenames = ""
         if elements:
@@ -345,13 +378,14 @@ class Winapp:
         removeself = False
         for element in elements:
             element = element.upper()
-            if 'RECURSE' == element:
+            if element == 'RECURSE':
                 recurse = True
-            elif 'REMOVESELF' == element:
+            elif element == 'REMOVESELF':
                 recurse = True
                 removeself = True
             else:
-                logger.warning('unknown file option %s in section %s', element, ini_section)
+                logger.warning(
+                    'unknown file option %s in section %s', element, ini_section)
         for filename in filenames.split(';'):
             for dirname in dirnames:
                 for provider in self.__make_file_provider(dirname, filename, recurse, removeself, excludekeys):
@@ -360,10 +394,11 @@ class Winapp:
 
     def handle_regkey(self, lid, ini_section, ini_option):
         """Parse a RegKey# option"""
-        elements = self.parser.get(ini_section, ini_option).decode(FSE).strip().split('|')
+        elements = self.parser.get(
+            ini_section, ini_option).strip().split('|')
         path = xml_escape(elements[0])
         name = ""
-        if 2 == len(elements):
+        if len(elements) == 2:
             name = 'name="%s"' % xml_escape(elements[1])
         action_str = '<option command="winreg" path="%s" %s/>' % (path, name)
         provider = Winreg(parseString(action_str).childNodes[0])
@@ -379,18 +414,21 @@ class Winapp:
 def list_winapp_files():
     """List winapp2.ini files"""
     for dirname in (bleachbit.personal_cleaners_dir, bleachbit.system_cleaners_dir):
-        fn = os.path.join(dirname, 'winapp2.ini')
-        if os.path.exists(fn):
-            yield fn
+        fname = os.path.join(dirname, 'winapp2.ini')
+        if os.path.exists(fname):
+            yield fname
 
 
-def load_cleaners():
+def load_cleaners(cb_progress=lambda x: None):
     """Scan for winapp2.ini files and load them"""
+    cb_progress(0.0)
     for pathname in list_winapp_files():
         try:
-            inicleaner = Winapp(pathname)
+            inicleaner = Winapp(pathname, cb_progress)
         except Exception as e:
-            logger.exception("Error reading winapp2.ini cleaner '%s'", pathname)
+            logger.exception(
+                "Error reading winapp2.ini cleaner '%s'", pathname)
         else:
             for cleaner in inicleaner.get_cleaners():
                 Cleaner.backends[cleaner.id] = cleaner
+        yield True
