@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2018 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -17,6 +17,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from bleachbit.FileUtilities import extended_path, extended_path_undo
 """
 ***
 *** Owner: Andrew Ziem
@@ -93,11 +94,11 @@ from win32api import (GetVolumeInformation, GetDiskFreeSpace,
 from win32file import (CreateFile, CreateFileW,
                        CloseHandle, GetDriveType,
                        GetFileSize, GetFileAttributesW,
+                       SetFileAttributesW,
                        DeviceIoControl, SetFilePointer,
-                       ReadFile, WriteFile,
+                       WriteFile,
                        LockFile, DeleteFile,
-                       SetEndOfFile, FlushFileBuffers,
-                       EncryptFile)
+                       SetEndOfFile, FlushFileBuffers)
 from winioctlcon import (FSCTL_GET_RETRIEVAL_POINTERS,
                          FSCTL_GET_VOLUME_BITMAP,
                          FSCTL_GET_NTFS_VOLUME_DATA,
@@ -113,17 +114,17 @@ from win32con import (FILE_ATTRIBUTE_ENCRYPTED,
                       FILE_ATTRIBUTE_COMPRESSED,
                       FILE_ATTRIBUTE_SPARSE_FILE,
                       FILE_ATTRIBUTE_HIDDEN,
+                      FILE_ATTRIBUTE_READONLY,
                       FILE_FLAG_RANDOM_ACCESS,
                       FILE_FLAG_NO_BUFFERING,
                       FILE_FLAG_WRITE_THROUGH,
                       COMPRESSION_FORMAT_DEFAULT)
 VER_SUITE_PERSONAL = 0x200   # doesn't seem to be present in win32con.
 
-from bleachbit.FileUtilities import extended_path, extended_path_undo
 
 # Constants.
 simulate_concurrency = False     # remove this test function when QA complete
-#drive_letter_safety = "E"       # protection to only use removeable drives
+# drive_letter_safety = "E"       # protection to only use removeable drives
 # don't use C: or D:, but E: and beyond OK.
 tmp_file_name = "bbtemp.dat"
 spike_file_name = "bbspike"     # cluster number will be appended
@@ -134,6 +135,8 @@ logger = logging.getLogger(__name__)
 
 # Unpacks the next element in a structure, using format requested.
 # Returns the element and the remaining content of the structure.
+
+
 def unpack_element(fmt, structure):
     chunk_size = struct.calcsize(fmt)
     element = struct.unpack(fmt, structure[:chunk_size])
@@ -280,7 +283,7 @@ def choose_if_bridged(volume_handle, total_clusters,
     extra_allocated_clusters = count_ballocated - count_oallocated
     saving_in_extents = len(orig_extents) - len(bridged_extents)
     logger.debug(("Bridged extents would require us to work around %d " +
-                   "more allocated clusters.") % extra_allocated_clusters)
+                  "more allocated clusters.") % extra_allocated_clusters)
     logger.debug("It would reduce extent count from %d to %d." % (
         len(orig_extents), len(bridged_extents)))
 
@@ -311,7 +314,7 @@ def split_extent(lcn_start, lcn_end):
     while count > split_factor**(exponent + 1.3):
         exponent += 1
     extent_size = split_factor**exponent
-    for x in xrange(lcn_start, lcn_end + 1, extent_size):
+    for x in range(lcn_start, lcn_end + 1, extent_size):
         yield (x, min(x + extent_size - 1, lcn_end))
 
 
@@ -319,7 +322,7 @@ def split_extent(lcn_start, lcn_end):
 def check_extents(extents, volume_bitmap, allocated_extents=None):
     count_free, count_allocated = (0, 0)
     for lcn_start, lcn_end in extents:
-        for cluster in xrange(lcn_start, lcn_end + 1):
+        for cluster in range(lcn_start, lcn_end + 1):
             if check_mapped_bit(volume_bitmap, cluster):
                 count_allocated += 1
                 if allocated_extents is not None:
@@ -328,7 +331,7 @@ def check_extents(extents, volume_bitmap, allocated_extents=None):
                 count_free += 1
 
     logger.debug("Extents checked: clusters free %d; allocated %d",
-                  count_free, count_allocated)
+                 count_free, count_allocated)
     return (count_free, count_allocated)
 
 
@@ -344,7 +347,7 @@ def check_extents_concurrency(extents, volume_bitmap,
 
     count_free, count_allocated = (0, 0)
     for lcn_start, lcn_end in extents:
-        for cluster in xrange(lcn_start, lcn_end + 1):
+        for cluster in range(lcn_start, lcn_end + 1):
             # Every once in a while, occupy a particular cluster on disk.
             if randint(1, odds_to_allocate) == odds_to_allocate:
                 spike_cluster(volume_handle, cluster, tmp_file_path)
@@ -366,7 +369,7 @@ def check_extents_concurrency(extents, volume_bitmap,
                 count_free += 1
 
     logger.debug("Extents checked: clusters free %d; allocated %d",
-                  count_free, count_allocated)
+                 count_free, count_allocated)
     return (count_free, count_allocated)
 
 
@@ -443,9 +446,9 @@ def get_file_basic_info(file_name, file_handle):
     is_special = is_compressed | is_encrypted | is_sparse
     if is_special:
         logger.debug('{}: {} {} {}'.format(file_name,
-            'compressed' if is_compressed else '',
-            'encrypted' if is_encrypted else '',
-            'sparse' if is_sparse else ''))
+                                           'compressed' if is_compressed else '',
+                                           'encrypted' if is_encrypted else '',
+                                           'sparse' if is_sparse else ''))
     return file_size, is_special
 
 
@@ -475,6 +478,8 @@ class UnsupportedFileSystemError(Exception):
 # First call: Drive Name; Max Path; File System.
 # Second call: Sectors per Cluster; Bytes per Sector; Total # of Clusters.
 # Third call: Drive Type.
+
+
 def get_volume_information(volume):
     # If it's a UNC path, raise an error.
     if not volume:
@@ -500,18 +505,18 @@ def get_volume_information(volume):
             "This file system (UDF) is not supported.")
 
     volume_info = namedtuple('VolumeInfo', [
-            'drive_name', 'max_path', 'file_system',
-            'sectors_per_cluster', 'bytes_per_sector', 'total_clusters'])
+        'drive_name', 'max_path', 'file_system',
+        'sectors_per_cluster', 'bytes_per_sector', 'total_clusters'])
 
     return volume_info(result1[0], result1[2], result1[4],
-            result2[0], result2[1], result2[3])
+                       result2[0], result2[1], result2[3])
 
 
 # Get read/write access to a volume.
 def obtain_readwrite(volume):
     # Optional protection that we are running on removable media only.
     assert volume
-    #if drive_letter_safety:
+    # if drive_letter_safety:
     #    drive_containing_file = volume[0].upper()
     #    assert drive_containing_file >= drive_letter_safety.upper()
 
@@ -551,7 +556,7 @@ def get_extents(file_handle, translate_to_extents=True):
                                         retrieval_pointers_buf_size)
         except:
             err_info = sys.exc_info()[1]
-            err_code, err_module, err_desc = err_info
+            err_code = err_info.winerror
             if err_code == 38:     # when file size is 0.
                 # (38, 'DeviceIoControl', 'Reached the end of the file.')
                 return []
@@ -702,7 +707,7 @@ def poll_clusters_freed(volume_handle, total_clusters, orig_extents):
     if not orig_extents:
         return True
 
-    for _ in xrange(polling_duration_seconds * attempts_per_second):
+    for _ in range(polling_duration_seconds * attempts_per_second):
         volume_bitmap, bitmap_size = get_volume_bitmap(volume_handle,
                                                        total_clusters)
         count_free, count_allocated = check_extents(
@@ -770,7 +775,7 @@ def wipe_file_direct(file_handle, extents, cluster_size, file_size):
     if extents:
         # Use size on disk to determine how many clusters of zeros we write.
         for lcn_start, lcn_end in extents:
-            #logger.debug("Wiping extent from %d to %d...",
+            # logger.debug("Wiping extent from %d to %d...",
             #              lcn_start, lcn_end)
             write_length = (lcn_end - lcn_start + 1) * cluster_size
             write_zero_fill(file_handle, write_length)
@@ -797,7 +802,7 @@ def wipe_extent_by_defrag(volume_handle, lcn_start, lcn_end, cluster_size,
                           total_clusters, tmp_file_path):
     assert cluster_size > 0
     logger.debug("Examining extent from %d to %d for wipe...",
-                  lcn_start, lcn_end)
+                 lcn_start, lcn_end)
     write_length = (lcn_end - lcn_start + 1) * cluster_size
 
     # Check the state of the volume bitmap for the extent we want to
@@ -849,7 +854,7 @@ def wipe_extent_by_defrag(volume_handle, lcn_start, lcn_end, cluster_size,
 
         if new_lcn_start != cluster_dest:
             logger.debug("Move %d clusters to %d",
-                          cluster_count, cluster_dest)
+                         cluster_count, cluster_dest)
             try:
                 move_file(volume_handle, file_handle, new_vcn,
                           cluster_dest, cluster_count)
@@ -914,6 +919,10 @@ def file_wipe(file_name):
     #logger.debug('Original extents: {}'.format(orig_extents))
 
     volume_handle = obtain_readwrite(volume)
+    attrs = GetFileAttributesW(file_name)
+    if attrs & FILE_ATTRIBUTE_READONLY:
+        # Remove read-only attribute to avoid "access denied" in CreateFileW().
+        SetFileAttributesW(file_name, attrs & ~FILE_ATTRIBUTE_READONLY)
     file_handle = open_file(file_name, GENERIC_READ | GENERIC_WRITE)
 
     if not is_special:
@@ -950,14 +959,13 @@ def file_wipe(file_name):
     tmp_file_path = os.path.dirname(file_name) + os.sep + tmp_file_name
     if is_special:
         orig_extents = choose_if_bridged(volume_handle,
-                                volume_info.total_clusters,
-                                orig_extents, bridged_extents)
+                                         volume_info.total_clusters,
+                                         orig_extents, bridged_extents)
     for lcn_start, lcn_end in orig_extents:
         result = wipe_extent_by_defrag(volume_handle, lcn_start, lcn_end,
-                                cluster_size, volume_info.total_clusters,
-                                tmp_file_path)
+                                       cluster_size, volume_info.total_clusters,
+                                       tmp_file_path)
 
     # Clean up.
     clean_up(None, volume_handle, tmp_file_path)
     return
-
