@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2019 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -21,7 +21,6 @@
 """
 Test cases for module CleanerML
 """
-from __future__ import absolute_import, print_function
 
 from tests import common
 from bleachbit.CleanerML import *
@@ -52,7 +51,6 @@ class CleanerMLTestCase(common.BleachbitTestCase):
         if common.destructive_tests('example_cleaner.xml'):
             self.run_all(xmlcleaner, True)
 
-
     def test_boolstr_to_bool(self):
         """Unit test for boolstr_to_bool()"""
         tests = [('True', True),
@@ -83,9 +81,10 @@ class CleanerMLTestCase(common.BleachbitTestCase):
 
         # should catch exception with invalid XML
         pcd = bleachbit.personal_cleaners_dir
-        bleachbit.personal_cleaners_dir = self.mkdtemp(prefix='bleachbit-cleanerml-load')
+        bleachbit.personal_cleaners_dir = self.mkdtemp(
+            prefix='bleachbit-cleanerml-load')
         self.write_file(os.path.join(bleachbit.personal_cleaners_dir, 'invalid.xml'),
-                        contents='<xml><broken>')
+                        contents=b'<xml><broken>')
         list(load_cleaners())
         import shutil
         shutil.rmtree(bleachbit.personal_cleaners_dir)
@@ -137,7 +136,7 @@ class CleanerMLTestCase(common.BleachbitTestCase):
 """.format(**{'tempdir': self.tempdir})
         # write XML cleaner
         cml_path = os.path.join(self.tempdir, 'test.xml')
-        self.write_file(cml_path, xml_str)
+        self.write_file(cml_path, xml_str.encode(sys.getdefaultencoding()))
 
         # create two canaries
         test_log_path_a = os.path.join(self.tempdir, 'a', 'test.log')
@@ -162,4 +161,3 @@ class CleanerMLTestCase(common.BleachbitTestCase):
         self.run_all(xmlc, True)
         self.assertNotExists(test_log_path_a)
         self.assertNotExists(test_log_path_b)
-
