@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2019 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,13 +23,11 @@
 Test case for module FileUtilities
 """
 
-from __future__ import absolute_import, print_function
-
 from tests import common
 from bleachbit.FileUtilities import *
 from bleachbit.General import run_external, sudo_mode
 from bleachbit.Options import options
-from bleachbit import expanduser, expandvars, logger
+from bleachbit import logger
 
 import json
 import sys
@@ -41,7 +39,6 @@ def test_ini_helper(self, execute):
 
     teststrings = [
         b'#Test\n[RecentsMRL]\nlist=C:\\Users\\me\\Videos\\movie.mpg,C:\\Users\\me\\movie2.mpg\n\n',
-        u'#Test\n[RecentsMRL]\nlist=C:\\Юзъри\\me\\Videos\\movie.mpg,C:\\Users\\me\\movie2.mpg\n\n'.encode('ISO-8859-5')
     ]
     for teststr in teststrings:
 
@@ -82,8 +79,9 @@ def test_json_helper(self, execute):
     expected = {'deleteme': 1, 'spareme': {'deletemetoo': 1}}
 
     # create test file
-    (fd, filename) = tempfile.mkstemp(prefix='bleachbit-test-json', dir=self.tempdir)
-    os.write(fd, '{ "deleteme" : 1, "spareme" : { "deletemetoo" : 1 } }')
+    (fd, filename) = tempfile.mkstemp(
+        prefix='bleachbit-test-json', dir=self.tempdir)
+    os.write(fd, b'{ "deleteme" : 1, "spareme" : { "deletemetoo" : 1 } }')
     os.close(fd)
     self.assertExists(filename)
     self.assertEqual(load_js(filename), expected)
@@ -119,9 +117,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
     def test_bytes_to_human(self):
         """Unit test for class bytes_to_human"""
 
-        if 'posix' == os.name:
-            old_locale = locale.getlocale(locale.LC_NUMERIC)
-            locale.setlocale(locale.LC_NUMERIC, 'en_US.UTF-8')
+        old_locale = locale.getlocale(locale.LC_NUMERIC)
+        locale.setlocale(locale.LC_NUMERIC, 'C')
 
         # test one-way conversion for predefined values
         # each test is a tuple in the format: (bytes, SI, EIC)
@@ -174,14 +171,13 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
                 self.assertEqual("1,01GB", bytes_to_human(1000 ** 3 + 5812389))
 
         # clean up
-        if 'posix' == os.name:
-            locale.setlocale(locale.LC_NUMERIC, old_locale)
+        locale.setlocale(locale.LC_NUMERIC, old_locale)
 
     def test_children_in_directory(self):
         """Unit test for function children_in_directory()"""
 
         # test an existing directory that usually exists
-        dirname = expanduser("~/.config")
+        dirname = os.path.expanduser("~/.config")
         for filename in children_in_directory(dirname, True):
             self.assertTrue(os.path.isabs(filename))
         for filename in children_in_directory(dirname, False):
@@ -203,7 +199,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         for filename in children_in_directory(dirname, True):
             self.assertEqual(filename, subdirname)
         for filename in children_in_directory(dirname, False):
-            raise AssertionError('Found a file that shouldn\'t have been found: ' + filename)
+            raise AssertionError(
+                'Found a file that shouldn\'t have been found: ' + filename)
         os.rmdir(subdirname)
 
         os.rmdir(dirname)
@@ -240,22 +237,30 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
 
     def test_delete_not_empty(self):
         """Test for scenario directory is not empty"""
-        print('hello')
-        fn =os.path.join(self.tempdir, 'make-directory-not-empty')
+        dirname = os.path.join(self.tempdir, 'a_dir')
+        os.mkdir(dirname)
+        self.assertTrue(is_dir_empty(dirname))
+        fn = os.path.join(dirname, 'a_file')
         common.touch_file(fn)
+        self.assertFalse(is_dir_empty(dirname))
         self.assertExists(fn)
+        self.assertExists(dirname)
         self.assertExists(self.tempdir)
-        delete(self.tempdir, allow_shred=False)
-        self.assertExists(fn)
-        self.assertExists(self.tempdir)
+
+        # Make sure shredding does not leave a renamed directory like
+        # in https://github.com/bleachbit/bleachbit/issues/783
+        for allow_shred in (False, True):
+            delete(dirname, allow_shred=allow_shred)
+            self.assertExists(fn)
+            self.assertExists(dirname)
 
     def delete_helper(self, shred):
         """Called by test_delete() with shred = False and = True"""
 
         # test deleting with various kinds of filenames
-        hebrew = u"עִבְרִית"
-        katanana = u"アメリカ"
-        umlauts = u"ÄäǞǟËëḦḧÏïḮḯÖöȪȫṎṏT̈ẗÜüǕǖǗǘǙǚǛǜṲṳṺṻẄẅẌẍŸÿ"
+        hebrew = "עִבְרִית"
+        katanana = "アメリカ"
+        umlauts = "ÄäǞǟËëḦḧÏïḮḯÖöȪȫṎṏT̈ẗÜüǕǖǗǘǙǚǛǜṲṳṺṻẄẅẌẍŸÿ"
 
         tests = ['.prefixandsuffix',  # simple
                  "x".zfill(150),  # long
@@ -263,9 +268,9 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
                  "''",  # quotation mark
                  "~`!@#$%^&()-_+=x",  # non-alphanumeric characters
                  "[]{};'.,x",  # non-alphanumeric characters
-                 u'abcdefgh',  # simple Unicode
-                 u'J\xf8rgen Scandinavian',
-                 u'\u2014em-dash',  # LP#1454030
+                 'abcdefgh',  # simple Unicode
+                 'J\xf8rgen Scandinavian',
+                 '\u2014em-dash',  # LP#1454030
                  hebrew,
                  katanana,
                  umlauts,
@@ -273,10 +278,10 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         if 'posix' == os.name:
             # Windows doesn't allow these characters but Unix systems do
             tests += ['"*', '\t\\', ':?<>|',
-                      ' ', '.file.'] # Windows filenames cannot end with space or period
+                      ' ', '.file.']  # Windows filenames cannot end with space or period
         for test in tests:
             # create the file
-            filename = self.write_file(test, "top secret")
+            filename = self.write_file(test, b"top secret")
             # delete the file
             delete(filename, shred)
             self.assertNotExists(filename)
@@ -291,7 +296,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
             if 'nt' == os.name:
                 from win32com.shell import shell
                 if not shell.IsUserAnAdmin():
-                    self.skipTest('skipping symlink test because of insufficient privileges')
+                    self.skipTest(
+                        'skipping symlink test because of insufficient privileges')
 
             # make regular file
             srcname = self.mkstemp(prefix='bleachbit-test-delete-regular')
@@ -344,10 +350,11 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
             kern = ctypes.windll.LoadLibrary("kernel32.dll")
 
             def win_symlink(src, linkname):
-                rc = kern.CreateSymbolicLinkA(linkname, src, 0)
+                rc = kern.CreateSymbolicLinkW(linkname, src, 0)
                 if rc == 0:
-                    print('CreateSymbolicLinkA(%s, %s)' % (linkname, src))
-                    print('CreateSymolicLinkA() failed, error = %s' % ctypes.FormatError())
+                    print('CreateSymbolicLinkW(%s, %s)' % (linkname, src))
+                    print('CreateSymolicLinkW() failed, error = %s' %
+                          ctypes.FormatError())
                     self.assertNotEqual(rc, 0)
             symlink_helper(win_symlink)
 
@@ -468,7 +475,18 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         msg = 'error calling umount\nargs=%s\nstderr=%s' % (args, stderr)
         self.assertEqual(rc, 0, msg)
 
-
+    def test_detect_encoding(self):
+        """Unit test for detect_encoding"""
+        eat_glass='나는 유리를 먹을 수 있어요. 그래도 아프지 않아요'
+        tests = (('This is just an ASCII file', 'ascii'),
+                 (eat_glass, 'utf-8'),
+                 (eat_glass, 'EUC-KR'))
+        for file_contents, expected_encoding in tests:
+            with tempfile.NamedTemporaryFile(mode='w', delete=False, encoding=expected_encoding) as temp:
+                temp.write(file_contents)
+                temp.flush()
+            det = detect_encoding(temp.name)
+            self.assertEqual(det, expected_encoding)
 
     @common.skipIfWindows
     def test_ego_owner(self):
@@ -505,15 +523,16 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
 
     def test_expandvars(self):
         """Unit test for expandvars()."""
-        expanded = expandvars('$HOME')
-        self.assertIsUnicodeString(expanded)
+        expanded = os.path.expandvars('$HOME')
+        self.assertIsString(expanded)
 
     def test_extended_path(self):
         """Unit test for extended_path() and extended_path_undo()"""
         if 'nt' == os.name:
             tests = [
                 (r'c:\windows\notepad.exe', r'\\?\c:\windows\notepad.exe'),
-                (r'\\server\share\windows\notepad.exe', r'\\?\unc\server\share\windows\notepad.exe'),
+                (r'\\server\share\windows\notepad.exe',
+                 r'\\?\unc\server\share\windows\notepad.exe'),
             ]
         else:
             # unchanged
@@ -529,7 +548,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
 
     def test_free_space(self):
         """Unit test for free_space()"""
-        home = expanduser('~')
+        home = os.path.expanduser('~')
         result = free_space(home)
         self.assertNotEqual(result, None)
         self.assertGreater(result, -1)
@@ -551,7 +570,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
             drive, bytes_free = re.split('\s+', line)
             print('Checking free space for %s' % drive)
             bytes_free = int(bytes_free)
-            free = free_space(unicode(drive))
+            free = free_space(drive)
             self.assertEqual(bytes_free, free)
 
     def test_getsize(self):
@@ -559,7 +578,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         dirname = self.mkdtemp(prefix='bleachbit-test-getsize')
 
         def test_getsize_helper(fname):
-            filename = self.write_file(os.path.join(dirname, fname), "abcdefghij" * 12345)
+            filename = self.write_file(os.path.join(
+                dirname, fname), b"abcdefghij" * 12345)
 
             if 'nt' == os.name:
                 self.assertEqual(getsize(filename), 10 * 12345)
@@ -576,8 +596,9 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
                     counter += 1
                 self.assertEqual(counter, 1)
             if 'posix' == os.name:
-                output = subprocess.Popen(
-                    ["du", "-h", filename], stdout=subprocess.PIPE).communicate()[0]
+                encoding = sys.getdefaultencoding()
+                output = str(subprocess.Popen(
+                    ["du", "-h", filename], stdout=subprocess.PIPE).communicate()[0], encoding=encoding)
                 output = output.replace("\n", "")
                 du_size = output.split("\t")[0] + "B"
                 print("output = '%s', size='%s'" % (output, du_size))
@@ -591,13 +612,13 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         test_getsize_helper('bleachbit-test-regular')
 
         # special characters
-        test_getsize_helper(u'bleachbit-test-special-characters-∺ ∯')
+        test_getsize_helper('bleachbit-test-special-characters-∺ ∯')
 
         # em-dash (LP1454030)
-        test_getsize_helper(u'bleachbit-test-em-dash-\u2014')
+        test_getsize_helper('bleachbit-test-em-dash-\u2014')
 
         # long
-        test_getsize_helper(u'bleachbit-test-long' + 'x' * 200)
+        test_getsize_helper('bleachbit-test-long' + 'x' * 200)
 
         # delete the empty directory
         delete(dirname)
@@ -607,12 +628,14 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
             return
 
         # create a symlink
-        filename = self.write_file('bleachbit-test-symlink', 'abcdefghij' * 12345)
+        filename = self.write_file(
+            'bleachbit-test-symlink', b'abcdefghij' * 12345)
         linkname = os.path.join(self.tempdir, 'bleachbitsymlinktest')
         if os.path.lexists(linkname):
             delete(linkname)
         os.symlink(filename, linkname)
-        self.assertLess(getsize(linkname), 8192, "Symlink size is %d" % getsize(filename))
+        self.assertLess(getsize(linkname), 8192,
+                        "Symlink size is %d" % getsize(filename))
         delete(filename)
 
         if 'darwin' == sys.platform:
@@ -641,7 +664,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
     def test_guess_overwrite_paths(self):
         """Unit test for guess_overwrite_paths()"""
         for path in guess_overwrite_paths():
-            self.assertTrue(os.path.isdir(path), '%s is not a directory' % path)
+            self.assertTrue(os.path.isdir(path),
+                            '%s is not a directory' % path)
 
     def test_human_to_bytes(self):
         """Unit test for human_to_bytes()"""
@@ -665,10 +689,10 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         """Unit test for listdir()"""
         if 'posix' == os.name:
             dir1 = '/bin'
-            dir2 = expanduser('/sbin')
+            dir2 = os.path.expanduser('/sbin')
         if 'nt' == os.name:
-            dir1 = expandvars(r'%windir%\fonts')
-            dir2 = expandvars(r'%windir%\logs')
+            dir1 = os.path.expandvars(r'%windir%\fonts')
+            dir2 = os.path.expandvars(r'%windir%\logs')
         # If these directories do not exist, the test results are not valid.
         self.assertExists(dir1)
         self.assertExists(dir2)
@@ -677,7 +701,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         paths2 = set(listdir(dir2))
         self.assertGreater(len(paths1), 4)
         self.assertGreater(len(paths2), 4)
-        paths12 = set(listdir((dir1,dir2)))
+        paths12 = set(listdir((dir1, dir2)))
         self.assertTrue(paths1 < paths12)
         self.assertTrue(paths2 < paths12)
         # The individual calls should be equivalent to a combined call.
@@ -691,7 +715,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
 
     def test_same_partition(self):
         """Unit test for same_partition()"""
-        home = expanduser('~')
+        home = os.path.expanduser('~')
         self.assertTrue(same_partition(home, home))
         if 'posix' == os.name:
             self.assertFalse(same_partition(home, '/dev'))
@@ -700,7 +724,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
             from bleachbit.Windows import get_fixed_drives
             for drive in get_fixed_drives():
                 this_drive = os.path.splitdrive(drive)[0]
-                self.assertEqual(same_partition(home, drive), home_drive == this_drive)
+                self.assertEqual(same_partition(home, drive),
+                                 home_drive == this_drive)
 
     def test_uris_to_paths(self):
         """Unit test for uris_to_paths()"""
@@ -708,12 +733,12 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
 
         # Unix-style
         uri_u = ['file:///usr/bin/bleachbit']
-        path_u = [os.path.normpath(u'/usr/bin/bleachbit'), ]
+        path_u = [os.path.normpath('/usr/bin/bleachbit'), ]
         self.assertEqual(uris_to_paths(uri_u), path_u)
 
         # Windows
         uri_w = [r'file:///C:/program%20files/bleachbit.exe']
-        path_w = [os.path.normpath(ur'C:/program files/bleachbit.exe'), ]
+        path_w = [os.path.normpath(r'C:/program files/bleachbit.exe'), ]
         self.assertEqual(uris_to_paths(uri_w), path_w)
 
         # Multiple
@@ -814,7 +839,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         d = '/usr/bin'
         whitelist = [('file', '/home/foo'), ('folder', '/home/folder')]
         if 'nt' == os.name:
-            d = expandvars('%windir%\system32')
+            d = os.path.expandvars('%windir%\system32')
             whitelist = [('file', r'c:\\filename'), ('folder', r'c:\\folder')]
         reps = 20
         paths = [p for p in children_in_directory(d, True)]
@@ -824,7 +849,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         options.set_whitelist_paths(whitelist)
 
         t0 = time.time()
-        for i in xrange(0, reps):
+        for i in range(0, reps):
             for p in paths:
                 _ = whitelisted(p)
         t1 = time.time()
@@ -837,7 +862,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         """Unit test for wipe_delete()"""
 
         # create test file
-        filename = self.write_file('bleachbit-test-wipe', 'abcdefghij' * 12345)
+        filename = self.write_file('bleachbit-test-wipe', b'abcdefghij' * 12345)
 
         # wipe it
         wipe_contents(filename)
@@ -846,9 +871,9 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         f = open(filename, 'rb')
         while True:
             byte = f.read(1)
-            if "" == byte:
+            if b"" == byte:
                 break
-            self.assertEqual(byte, chr(0))
+            self.assertEqual(byte, 0)
         f.close()
 
         # clean up
@@ -934,7 +959,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         def number_generator():
             for x in range(1, 10000):
                 yield (x, )
-        conn.executemany('insert into numbers (number) values ( ? ) ', number_generator())
+        conn.executemany(
+            'insert into numbers (number) values ( ? ) ', number_generator())
         conn.commit()
         self.assertLess(empty_size, getsize(path))
         conn.execute('delete from numbers')
@@ -970,4 +996,5 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         self.assertFalse(openfiles.is_open(filename))
 
     def test_open_files_lsof(self):
-        self.assertEqual(list(open_files_lsof(lambda: 'n/bar/foo\nn/foo/bar\nnoise')), ['/bar/foo', '/foo/bar'])
+        self.assertEqual(list(open_files_lsof(
+            lambda: 'n/bar/foo\nn/foo/bar\nnoise')), ['/bar/foo', '/foo/bar'])
