@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2019 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,8 +23,6 @@
 Test case for module Unix
 """
 
-from __future__ import absolute_import
-
 from tests import common
 from bleachbit.Unix import *
 
@@ -96,7 +94,8 @@ root               531   0.0  0.0  2501712    588   ??  Ss   20May16   0:02.40 s
 """
         self.assertTrue(is_running_darwin('USBAgent', run_ps))
         self.assertFalse(is_running_darwin('does-not-exist', run_ps))
-        self.assertRaises(RuntimeError, is_running_darwin, 'foo', lambda: 'invalid-input')
+        self.assertRaises(RuntimeError, is_running_darwin,
+                          'foo', lambda: 'invalid-input')
 
     @common.skipIfWindows
     def test_is_running(self):
@@ -136,7 +135,8 @@ root               531   0.0  0.0  2501712    588   ??  Ss   20May16   0:02.40 s
             self.assertIsNotNone(m, 'expected positive match for ' + locale)
             self.assertEqual(m.group("locale"), tlc)
         for test in ['default', 'C', 'English', 'ru_RU.txt', 'ru.txt']:
-            self.assertIsNone(regex.match(test), 'expected negative match for ' + test)
+            self.assertIsNone(regex.match(
+                test), 'expected negative match for ' + test)
 
     @common.skipIfWindows
     def test_localization_paths(self):
@@ -148,7 +148,7 @@ root               531   0.0  0.0  2501712    588   ??  Ss   20May16   0:02.40 s
         counter = 0
         for path in locales.localization_paths(['en', 'en_AU', 'en_CA', 'en_GB']):
             self.assertLExists(path)
-            # self.assert_(path.startswith('/usr/share/locale'))
+            # self.assertTrue(path.startswith('/usr/share/locale'))
             # /usr/share/locale/en_* should be ignored
             self.assertEqual(path.find('/en_'), -1)
             counter += 1
@@ -195,7 +195,8 @@ root               531   0.0  0.0  2501712    588   ??  Ss   20May16   0:02.40 s
         self.locales._paths = LocaleCleanerPath(self.tempdir)
         self.locales.add_xml(config.firstChild, None)
         # normpath because paths may contain ./
-        deletelist = [os.path.normpath(path) for path in self.locales.localization_paths(['en', 'de'])]
+        deletelist = [os.path.normpath(
+            path) for path in self.locales.localization_paths(['en', 'de'])]
         for path in keepdirs + keepfiles:
             self.assertNotIn(os.path.join(self.tempdir, path), deletelist)
         for path in nukedirs + nukefiles:
@@ -205,13 +206,16 @@ root               531   0.0  0.0  2501712    588   ??  Ss   20May16   0:02.40 s
     def test_rotated_logs(self):
         """Unit test for rotated_logs()"""
         for path in rotated_logs():
-            self.assertLExists(path, "Rotated log path '%s' does not exist" % path)
+            self.assertLExists(
+                path, "Rotated log path '%s' does not exist" % path)
 
     @common.skipIfWindows
     def test_run_cleaner_cmd(self):
         from subprocess import CalledProcessError
-        self.assertRaises(RuntimeError, run_cleaner_cmd, '/hopethisdoesntexist', [])
-        self.assertRaises(CalledProcessError, run_cleaner_cmd, 'sh', ['-c', 'echo errormsg; false'])
+        self.assertRaises(RuntimeError, run_cleaner_cmd,
+                          '/hopethisdoesntexist', [])
+        self.assertRaises(CalledProcessError, run_cleaner_cmd,
+                          'sh', ['-c', 'echo errormsg; false'])
         # test if regexes for invalid lines work
         self.assertRaises(RuntimeError, run_cleaner_cmd, 'echo', ['This is an invalid line'],
                           error_line_regexes=['invalid'])
@@ -221,7 +225,8 @@ root               531   0.0  0.0  2501712    588   ??  Ss   20May16   0:02.40 s
                  'Freed 100B on your hard drive',
                  'Freed 1.9kB, hooray!',
                  'Fred 12MB']
-        freed_space = run_cleaner_cmd('echo', ['\n'.join(lines)], freed_space_regex)
+        freed_space = run_cleaner_cmd(
+            'echo', ['\n'.join(lines)], freed_space_regex)
         self.assertEqual(freed_space, 2000)
 
     @common.skipIfWindows
@@ -230,7 +235,8 @@ root               531   0.0  0.0  2501712    588   ??  Ss   20May16   0:02.40 s
         wineprefix = "/home/foo/.wine"
         windows_pathname = "C:\\Program Files\\NSIS\\NSIS.exe"
         result = "/home/foo/.wine/drive_c/Program Files/NSIS/NSIS.exe"
-        self.assertEqual(wine_to_linux_path(wineprefix, windows_pathname), result)
+        self.assertEqual(wine_to_linux_path(
+            wineprefix, windows_pathname), result)
 
     @common.skipIfWindows
     def test_yum_clean(self):
@@ -242,3 +248,25 @@ root               531   0.0  0.0  2501712    588   ??  Ss   20May16   0:02.40 s
             bytes_freed = yum_clean()
             self.assertIsInteger(bytes_freed)
             bleachbit.logger.debug('yum bytes cleaned %d', bytes_freed)
+
+    @common.skipIfWindows
+    def test_dnf_clean(self):
+        """Unit test for dnf_clean()"""
+        if 0 != os.geteuid() or os.path.exists('/var/run/dnf.pid') \
+                or not FileUtilities.exe_exists('dnf'):
+            self.assertRaises(RuntimeError, dnf_clean)
+        else:
+            bytes_freed = dnf_clean()
+            self.assertIsInteger(bytes_freed)
+            bleachbit.logger.debug('dnf bytes cleaned %d', bytes_freed)
+
+    @common.skipIfWindows
+    def test_dnf_autoremove(self):
+        """Unit test for dnf_autoremove()"""
+        if 0 != os.geteuid() or os.path.exists('/var/run/dnf.pid') \
+                or not FileUtilities.exe_exists('dnf'):
+            self.assertRaises(RuntimeError, dnf_clean)
+        else:
+            bytes_freed = dnf_autoremove()
+            self.assertIsInteger(bytes_freed)
+            bleachbit.logger.debug('dnf bytes cleaned %d', bytes_freed)
