@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2019 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,30 +23,30 @@
 Test case for module Windows
 """
 
-from __future__ import absolute_import, print_function
-
 from tests import common
 
 from bleachbit.FileUtilities import extended_path, extended_path_undo
 from bleachbit.Windows import *
 from bleachbit import logger
 
-import sys
+import functools
 import os
+import platform
+import shutil
+import sys
 import tempfile
 import unittest
-import platform
 from decimal import Decimal
 
 if 'win32' == sys.platform:
-    import _winreg
+    import winreg
     from win32com.shell import shell
 
 
 def put_files_into_recycle_bin():
     """Put a file and a folder into the recycle bin"""
     # make a file and move it to the recycle bin
-    tests = ('regular', u'unicode-emdash-u\u2014', 'long' + 'x' * 100)
+    tests = ('regular', 'unicode-emdash-u\u2014', 'long' + 'x' * 100)
     for test in tests:
         (fd, filename) = tempfile.mkstemp(
             prefix='bleachbit-recycle-file', suffix=test)
@@ -63,6 +63,10 @@ class WindowsTestCase(common.BleachbitTestCase):
 
     """Test case for module Windows"""
 
+    def skipUnlessAdmin(self):
+        if not shell.IsUserAnAdmin():
+            self.skipTest('requires administrator privileges')
+
     def test_get_recycle_bin(self):
         """Unit test for get_recycle_bin"""
         for f in get_recycle_bin():
@@ -80,33 +84,67 @@ class WindowsTestCase(common.BleachbitTestCase):
         for f in get_recycle_bin():
             self.fail('recycle bin should be empty, but it is not')
 
-    def test_link(self):
-        """Unit test for links with is_link() and get_recycle_bin()"""
-        if not common.destructive_tests('windows link'):
-            return
+    def _test_link_helper(self, mklink_option, clear_recycle_bin):
+        """Helper function for testing for links with is_junction() and
+        get_recycle_bin()
 
+        It gets called four times for the combinations of the two
+        parameters. It's called by four unit tests four accounting
+        purposes. In other words, we don't want to count a test as
+        skipped if part of it succeeded.
+
+        mklink /j = directory junction
+        directory junction does not require administrator privileges
+
+        mklink /d=directory symbolic link
+        requires administrator privileges
+        """
+        if mklink_option == '/d':
+            self.skipUnlessAdmin()
         # make a normal directory with a file in it
-        real_dir = os.path.join(self.tempdir, 'real_dir')
-        os.mkdir(real_dir)
-        self.assertExists(real_dir)
-        self.assertEqual(False, is_link(real_dir))
+        target_dir = os.path.join(self.tempdir, 'target_dir')
+        os.mkdir(target_dir)
+        self.assertExists(target_dir)
+        self.assertFalse(is_junction(target_dir))
 
-        canary_fn = os.path.join(real_dir, 'do_not_delete')
+        from random import randint
+        canary_fn = os.path.join(target_dir, 'do_not_delete%d' % randint(1000,9999))
         common.touch_file(canary_fn)
         self.assertExists(canary_fn)
-        self.assertEqual(False, is_link(canary_fn))
+        self.assertFalse(is_junction(canary_fn))
 
-        # link to the normal directory
-        link_dir = os.path.join(self.tempdir, 'link_dir')
-        args = ('cmd', '/c', 'mklink', '/d', link_dir, real_dir)
+        # make a normal directory to hold a link
+        container_dir = os.path.join(self.tempdir, 'container_dir')
+        os.mkdir(container_dir)
+        self.assertExists(container_dir)
+        self.assertFalse(is_junction(container_dir))
+
+        # create the link
+        link_pathname = os.path.join(container_dir, 'link')
+        args = ('cmd', '/c', 'mklink', mklink_option, link_pathname, target_dir)
         from bleachbit.General import run_external
         (rc, stdout, stderr) = run_external(args)
         self.assertEqual(rc, 0, stderr)
-        self.assertExists(link_dir)
-        self.assertEqual(True, is_link(link_dir))
+        self.assertExists(link_pathname)
+        self.assertTrue(is_junction(link_pathname))
 
         # put the link in the recycle bin
-        move_to_recycle_bin(link_dir)
+        move_to_recycle_bin(container_dir)
+
+        # preview the recycle bin
+        for f in get_recycle_bin():
+            print(f)
+
+        def cleanup_dirs():
+            shutil.rmtree(container_dir, True)
+            self.assertNotExists(container_dir)
+            shutil.rmtree(target_dir, True)
+
+        if not clear_recycle_bin:
+            cleanup_dirs()
+            if not common.destructive_tests('windows link'):
+                self.skipTest('destructive tests are disabled')
+            return
 
         # clear the recycle bin
         for f in get_recycle_bin():
@@ -115,9 +153,30 @@ class WindowsTestCase(common.BleachbitTestCase):
         # verify the canary is still there
         self.assertExists(canary_fn)
 
+        # clean up
+        cleanup_dirs()
+
+    @unittest.skip("not yet")
+    def test_link_junction_no_clear(self):
+        """Unit test for directory junctions without clearing recycle bin"""
+        self._test_link_helper('/j', False)
+
+    @unittest.skip("not yet")
+    def test_link_junction_clear(self):
+        """Unit test for directory junctions with clearing recycle bin"""
+        self._test_link_helper('/j', True)
+
+    def test_link_symlink_no_clear(self):
+        """Unit test for directory symlink without clearing recycle bin"""
+        self._test_link_helper('/d', False)
+
+    def test_link_symlink_clear(self):
+        """Unit test for directory symlink with clearing recycle bin"""
+        self._test_link_helper('/d', True)
+
     def test_delete_locked_file(self):
         """Unit test for delete_locked_file"""
-        tests = ('regular', u'unicode-emdash-u\u2014', 'long' + 'x' * 100)
+        tests = ('regular', 'unicode-emdash-u\u2014', 'long' + 'x' * 100)
         for test in tests:
             f = tempfile.NamedTemporaryFile(
                 prefix='bleachbit-delete-locked-file', suffix=test,
@@ -125,7 +184,7 @@ class WindowsTestCase(common.BleachbitTestCase):
             pathname = f.name
             f.close()
             import time
-            time.sleep(5) # avoid race condition
+            time.sleep(5)  # avoid race condition
             self.assertExists(pathname)
             logger.debug('delete_locked_file(%s) ' % pathname)
             if not shell.IsUserAnAdmin():
@@ -135,7 +194,8 @@ class WindowsTestCase(common.BleachbitTestCase):
                 try:
                     delete_locked_file(pathname)
                 except WindowsError:
-                    logger.exception('delete_locked_file() threw an error, which may be a false positive')
+                    logger.exception(
+                        'delete_locked_file() threw an error, which may be a false positive')
             self.assertExists(pathname)
         logger.info('reboot Windows and check the three files are deleted')
 
@@ -150,7 +210,7 @@ class WindowsTestCase(common.BleachbitTestCase):
         # create a nested key
         key = 'Software\\BleachBit\\DeleteThisKey'
         subkey = key + '\\AndThisKey'
-        hkey = _winreg.CreateKey(_winreg.HKEY_CURRENT_USER, subkey)
+        hkey = winreg.CreateKey(winreg.HKEY_CURRENT_USER, subkey)
         hkey.Close()
 
         # test
@@ -168,12 +228,12 @@ class WindowsTestCase(common.BleachbitTestCase):
         # UnicodeDecodeError: 'ascii' codec can't decode byte 0xc3 in position
         # 11: ordinal not in range(128)
         key = r'Software\\BleachBit\\DeleteThisKey'
-        hkey = _winreg.CreateKey(
-            _winreg.HKEY_CURRENT_USER, key + r'\\AndThisKey-Ö')
+        hkey = winreg.CreateKey(
+            winreg.HKEY_CURRENT_USER, key + r'\\AndThisKey-Ö')
         hkey.Close()
-        return_value = delete_registry_key(u'HKCU\\' + key, True)
+        return_value = delete_registry_key('HKCU\\' + key, True)
         self.assertTrue(return_value)
-        return_value = delete_registry_key(u'HKCU\\' + key, True)
+        return_value = delete_registry_key('HKCU\\' + key, True)
         self.assertFalse(return_value)
 
     def test_delete_registry_value(self):
@@ -185,11 +245,11 @@ class WindowsTestCase(common.BleachbitTestCase):
 
         # create a name-value pair
         key = 'Software\\BleachBit'
-        hkey = _winreg.CreateKey(_winreg.HKEY_CURRENT_USER, key)
+        hkey = winreg.CreateKey(winreg.HKEY_CURRENT_USER, key)
 
         value_name = 'delete_this_value_name'
-        _winreg.SetValueEx(
-            hkey, value_name, 0, _winreg.REG_SZ, 'delete this value')
+        winreg.SetValueEx(
+            hkey, value_name, 0, winreg.REG_SZ, 'delete this value')
         hkey.Close()
 
         # delete and confirm
@@ -216,8 +276,8 @@ class WindowsTestCase(common.BleachbitTestCase):
 
     def test_detect_registry_key(self):
         """Test for detect_registry_key()"""
-        self.assert_(detect_registry_key('HKCU\\Software\\Microsoft\\'))
-        self.assert_(not detect_registry_key('HKCU\\Software\\DoesNotExist'))
+        self.assertTrue(detect_registry_key('HKCU\\Software\\Microsoft\\'))
+        self.assertTrue(not detect_registry_key('HKCU\\Software\\DoesNotExist'))
 
     def test_get_clipboard_paths(self):
         """Unit test for get_clipboard_paths"""
@@ -235,7 +295,7 @@ class WindowsTestCase(common.BleachbitTestCase):
         win32clipboard.SetClipboardText(fname, win32clipboard.CF_TEXT)
         win32clipboard.SetClipboardText(fname, win32clipboard.CF_UNICODETEXT)
         self.assertEqual(win32clipboard.GetClipboardData(
-            win32clipboard.CF_TEXT), fname)
+            win32clipboard.CF_TEXT), fname.encode('ascii'))
         self.assertEqual(win32clipboard.GetClipboardData(
             win32clipboard.CF_UNICODETEXT), fname)
         win32clipboard.CloseClipboard()
@@ -287,18 +347,18 @@ class WindowsTestCase(common.BleachbitTestCase):
         # check the function basically works
         for drive in get_fixed_drives():
             ret = empty_recycle_bin(drive, really_delete=False)
-            self.assertIsInstance(ret, (int, long))
+            self.assertIsInteger(ret)
         if not common.destructive_tests('recycle bin'):
             return
         # check it deletes files for fixed drives
         put_files_into_recycle_bin()
         for drive in get_fixed_drives():
             ret = empty_recycle_bin(drive, really_delete=True)
-            self.assertIsInstance(ret, (int, long))
+            self.assertIsInteger(ret)
         # check it deletes files for all drives
         put_files_into_recycle_bin()
         ret = empty_recycle_bin(None, really_delete=True)
-        self.assertIsInstance(ret, (int, long))
+        self.assertIsInteger(ret)
         # Repeat two for reasons.
         # 1. Trying to empty an empty recycling bin can cause
         #    a 'catastrophic failure' error (handled in the function)
@@ -317,7 +377,7 @@ class WindowsTestCase(common.BleachbitTestCase):
 
         dirname = tempfile.mkdtemp(prefix='bleachbit-file-wipe')
 
-        filenames = ('short', 'long' + 'x' * 250, u'utf8-ɡælɪk')
+        filenames = ('short', 'long' + 'x' * 250, 'utf8-ɡælɪk')
         for filename in filenames:
             longname = os.path.join(dirname, filename)
             logger.debug('file_wipe(%s)', longname)
@@ -347,18 +407,19 @@ class WindowsTestCase(common.BleachbitTestCase):
                 self.assertNotExists(shortname)
 
             # A small file that fits in MFT
-            _test_wipe('')
+            _test_wipe(b'')
 
             # requires wiping of extents
-            _test_wipe('secret' * 100000)
+            _test_wipe(b'secret' * 100000)
 
-        import shutil
         shutil.rmtree(dirname, True)
 
         if shell.IsUserAnAdmin():
-            logger.warning('You should also run test_file_wipe() without admin privileges.')
+            logger.warning(
+                'You should also run test_file_wipe() without admin privileges.')
         else:
-            logger.warning('You should also run test_file_wipe() with admin privileges.')
+            logger.warning(
+                'You should also run test_file_wipe() with admin privileges.')
 
     def test_is_process_running(self):
         # winlogon.exe runs on Windows XP and Windows 7
@@ -380,13 +441,13 @@ class WindowsTestCase(common.BleachbitTestCase):
         if parse_windows_build() >= 6.0:
             envs.append('localappdatalow')
         for env in envs:
-            self.assertExists(os.environ[env].decode('utf8'))
+            self.assertExists(os.environ[env])
 
     def test_split_registry_key(self):
         """Unit test for split_registry_key"""
-        tests = (('HKCU\\Software', _winreg.HKEY_CURRENT_USER, 'Software'),
-                 ('HKLM\\SOFTWARE', _winreg.HKEY_LOCAL_MACHINE, 'SOFTWARE'),
-                 ('HKU\\.DEFAULT', _winreg.HKEY_USERS, '.DEFAULT'))
+        tests = (('HKCU\\Software', winreg.HKEY_CURRENT_USER, 'Software'),
+                 ('HKLM\\SOFTWARE', winreg.HKEY_LOCAL_MACHINE, 'SOFTWARE'),
+                 ('HKU\\.DEFAULT', winreg.HKEY_USERS, '.DEFAULT'))
         for (input_key, expected_hive, expected_key) in tests:
             (hive, key) = split_registry_key(input_key)
             self.assertEqual(expected_hive, hive)
@@ -418,12 +479,10 @@ class WindowsTestCase(common.BleachbitTestCase):
         self.assertEqual(ret, 0)
 
     def test_set_environ(self):
-        for folder in [u'folderäö', 'folder']:
+        for folder in ['folderäö', 'folder']:
             test_dir = os.path.join(self.tempdir, folder)
             os.mkdir(test_dir)
             self.assertExists(test_dir)
             set_environ('cd_test', test_dir)
-            if isinstance(test_dir, unicode):
-                test_dir = test_dir.encode('utf-8')
             self.assertEqual(os.environ['cd_test'], test_dir)
             os.environ.pop('cd_test')
