@@ -2,7 +2,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2019 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -22,8 +22,6 @@
 Test FileUtilities.wipe_path
 """
 
-from __future__ import absolute_import, print_function
-
 from bleachbit.FileUtilities import delete, free_space, listdir, wipe_path
 from bleachbit.General import run_external
 from tests import common
@@ -44,7 +42,7 @@ def create_disk_image(n_bytes):
     (fd, filename) = tempfile.mkstemp(
         suffix='disk-image', prefix='bleachbit-wipe-test')
     for x in range(1, int(n_bytes / 1e5)):
-        os.write(fd, '\x00' * 100000)
+        os.write(fd, b'\x00' * 100000)
     os.close(fd)
     return filename
 
@@ -85,7 +83,8 @@ def make_dirty(mountpoint):
         except:
             logger.error('while closing temporary file %s', f.name)
             break
-    logger.debug('created %d files and wrote to %d files', create_counter, write_counter)
+    logger.debug('created %d files and wrote to %d files',
+                 create_counter, write_counter)
 
 
 def mount_filesystem(filename, mountpoint):
@@ -118,7 +117,8 @@ def verify_cleanliness(filename):
     strings_ret = run_external(['strings', filename])
     secret_count = strings_ret[1].count('secret')  # filename
     sssshhhh_count = strings_ret[1].count('sssshhhh')  # contents
-    logger.debug('found %d sssshhhhh in image (contents) and %d secret (filename)', sssshhhh_count, secret_count)
+    logger.debug('found %d sssshhhhh in image (contents) and %d secret (filename)',
+                 sssshhhh_count, secret_count)
 
     clean = ((secret_count > 0) * 1) + ((sssshhhh_count > 0) * 10)
     print('%s is clean: %s', filename, clean)
@@ -227,4 +227,5 @@ def test_wipe():
             print(sys.exc_info()[1])
             traceback.print_exc()
 
+
 test_wipe()
