@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2019 Andrew Ziem
+# Copyright (C) 2008-2020 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -22,8 +22,6 @@
 Common code for unit tests
 """
 
-from __future__ import absolute_import, print_function
-
 from bleachbit.FileUtilities import extended_path
 from bleachbit.General import sudo_mode
 
@@ -59,36 +57,46 @@ class BleachbitTestCase(unittest.TestCase):
     # type asserts
     #
     def assertIsInteger(self, obj, msg=''):
-        self.assertIsInstance(obj, (int, long), msg)
-
-    def assertIsUnicodeString(self, obj, msg=''):
-        self.assertIsInstance(obj, unicode, msg)
+        self.assertIsInstance(obj, int, msg)
 
     def assertIsString(self, obj, msg=''):
-        self.assertIsInstance(obj, (unicode, str), msg)
+        self.assertIsInstance(obj, str, msg)
 
     def assertIsBytes(self, obj, msg=''):
         self.assertIsInstance(obj, bytes, msg)
 
+    @staticmethod
+    def check_exists(func, path):
+        try:
+            func(path)
+            return True
+        except PermissionError:
+            # Python 3.4: on Windows os.path.[l]exists may return False when access is denied:
+            # https://bugs.python.org/issue28075
+            return True
+        except:
+            return False
+
     #
     # file asserts
     #
-    def assertExists(self, path, msg='', func=os.path.exists):
+    def assertExists(self, path, msg='', func=os.stat):
         """File, directory, or any path exists"""
-        from bleachbit import expandvars
-        path = expandvars(path)
-        if not func(getTestPath(path)):
-            raise AssertionError('The file %s should exist, but it does not. %s' % (path, msg))
+        path = os.path.expandvars(path)
+        if not self.check_exists(func, getTestPath(path)):
+            raise AssertionError(
+                'The file %s should exist, but it does not. %s' % (path, msg))
+
+    def assertNotExists(self, path, msg='', func=os.stat):
+        if self.check_exists(func, getTestPath(path)):
+            raise AssertionError(
+                'The file %s should not exist, but it does. %s' % (path, msg))
 
     def assertLExists(self, path, msg=''):
-        self.assertExists(path, msg, os.path.lexists)
+        self.assertExists(path, msg, os.lstat)
 
     def assertNotLExists(self, path, msg=''):
-        self.assertNotExists(path, msg, os.path.lexists)
-
-    def assertNotExists(self, path, msg='', func=os.path.exists):
-        if func(getTestPath(path)):
-            raise AssertionError('The file %s should not exist, but it does. %s' % (path, msg))
+        self.assertNotExists(path, msg, os.lstat)
 
     def assertCondExists(self, cond, path, msg=''):
         if cond:
@@ -99,7 +107,7 @@ class BleachbitTestCase(unittest.TestCase):
     #
     # file creation functions
     #
-    def write_file(self, filename, contents=''):
+    def write_file(self, filename, contents=b''):
         """Create a temporary file, optionally writing contents to it"""
         if not os.path.isabs(filename):
             filename = os.path.join(self.tempdir, filename)
@@ -186,18 +194,17 @@ def validate_result(self, result, really_delete=False):
     self.assertLessEqual(result['n_deleted'], 1)
     self.assertEqual(result['n_special'] + result['n_deleted'], 1)
     # size
-    self.assertIsInstance(result['size'], (int, long, type(None),), "size is %s" % str(result['size']))
+    self.assertIsInstance(result['size'], (int, type(
+        None),), "size is %s" % str(result['size']))
     # path
     filename = result['path']
     if not filename:
         # the process action, for example, does not have a filename
         return
-    self.assertIsInstance(filename, (str, unicode, type(None)),
-                 "Filename is invalid: '%s' (type %s)" % (filename, type(filename)))
-    if isinstance(filename, (str, unicode)) and not filename[0:2] == 'HK':
+    self.assertIsInstance(filename, (str, type(None)),
+                          "Filename is invalid: '%s' (type %s)" % (filename, type(filename)))
+    if isinstance(filename, str) and not filename[0:2] == 'HK':
         if really_delete:
             self.assertNotLExists(filename)
         else:
             self.assertLExists(filename)
-
-
